/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

/**
 *
 * @author Jon
 */
import java.io.*;
import java.util.*;

public class ErrorUtility {


    //-------------------------------------------------------------------
    // void calcAndPrintRegionErrors
    //
    // 1) Calculate the base error, which is the error if all regions are
    //    set to 0
    //    - output to log
    //
    // 2) Calculate the SSE and SSE_Reduction for each and every region
    //    - output to log
    //
    // 3) Sort SSE_Reduction for all regions and calculate SSE for:
    //     a) Best 1
    //     b) Best 2
    //     ...
    //
    //
    //--------------------------------------------------------------------
    public static void calcAndPrintRegionErrorsFloat_ALL_COMBINATIONS(
                                          float [][][] initArr,
                                          float ssError[],
                                          float ssErrorReduct[],
                                          float transformedArr[][][],
                                          float compressedArr[][][],
                                          int dim1, int dim2, int dim3,
                                          String fileName,
                                          float range, BufferedWriter log,
                                          BufferedWriter log2,
                                          float unh_region_error[][],
                                          int num_lvls,
                                          boolean compressSummary,
                                          boolean compressDetails,
                                          int num_bins,
                                          String mapFilename
                                         )
    {
      

       // CREATE TEMPORARY ARRAYS
       float [][][] arr1 = new float[dim1][dim2][dim3];
       float [][][] arr2 = new float[dim1][dim2][dim3];

       // array NOT to be modified - holds all coefficients
       float [][][] permArr = new float[dim1][dim2][dim3];

       // Array to be modified based on regions being kept
       float [][][] tempArr = new float[dim1][dim2][dim3];

       // copy initArr for transform
       ArrayUtil.copyArray(arr1, initArr);

       // each iteration is a transform
        // conversions done in power of two
       int loop_lvls =   (int)Math.pow(2,(num_lvls-1));
       for (int xferLevel=1; xferLevel<=loop_lvls; xferLevel*=2) {
          // perform transform
          HaarTransform.transform(arr1, arr2, dim1/xferLevel,
                                  dim2/xferLevel, dim3/xferLevel);
          ArrayUtil.copyArray(arr1, arr2);
       }

       // save transformed array into permArr
       ArrayUtil.copyArray(permArr, arr1);

       ///////////////////////////////////////////////////////////////////
       ///////////////////////////////////////////////////////////////////
       //                                              ///////////////////
       // Calculate Error per combination!!!           ///////////////////
       //                                              ///////////////////
       ///////////////////////////////////////////////////////////////////
       ///////////////////////////////////////////////////////////////////
       int index =0;
       int lvl=num_lvls;
       int xferLevel=loop_lvls;
       String bestString = null;

       // number assigned per region SSS is region 0
       String [] arr = {"0","1","2","3","4","5","6","7"};
       String [] combinations;

       // Generate histograms
       float summaryMap[][]=null;
       float detailMap[][]=null;

       /* generate and test combinations for n=7, r=i*/
       for (int i=1;i<=arr.length;i++) {
       //for (int i=1;i<2;i++) {

           // generate combinations
           combinations = CombinationGenerator.generateCombinations( arr, i, "" );

           try {
              log.write("*** COMBINATIONS OF " + i + "***\n");
           }catch(Exception e) {
              System.out.println("Exception caught writing to log file." +
                                    "Aborting!!");
              System.exit(0);
           }

           // keep track of lowest error, and its region, for
           // - keeping 1 region, 2 regions ...
           float lowest= Float.MAX_VALUE;
           int lowest_index=0;
         
           // walk thru each combination
           for (int j=0; j<combinations.length; j++) {

              System.out.println("************** COMBO = "+combinations[j]);
              // initialize arr1 to zeros
              ArrayUtil.setFloatArray(arr1, 0.0f);
              ArrayUtil.setFloatArray(tempArr, 0.0f);

              // copy in regions we want to keep
              for (int k=0; k<=7; k++) {
                 // copy over regions in this combination!!
                 if ( combinations[j].contains(""+k))
                 {
                    // copy region into arr1
                    Region.copyRegionValues(arr1, permArr,
                                           Region.region.getRegion(k),num_lvls);
                    // only copy into tempArr if it is to be compressed
                    if ( (k==0 && compressSummary == true)
                         || (k!=0 && compressDetails == true )) 
                    {
                       Region.copyRegionValues(tempArr, permArr, 
                                           Region.region.getRegion(k),num_lvls);    
                    }
                 }
              }
              //!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
              // now arr1 holds all regions in combo, tempArr only holds
              // regions to be compressed
              //!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

              // compress regions we are keeping
              if ( (compressSummary && combinations[j].contains("0"))
                   || (compressDetails && comboContainsDetail(combinations[j])))
              {
                 compressCoefficients(tempArr, compressSummary,  compressDetails,
                               num_lvls, loop_lvls, num_bins,
                               summaryMap,
                               detailMap,
                               mapFilename);
              }

              // copy compressed summary if SSS is in combo and we were
              // told to compress summary
              if( compressSummary && (combinations[j].contains("0") ) ) {
                  Region.copyRegionValues(arr1, tempArr, Region.region.getRegion(0), num_lvls);
              }

              if ( compressDetails ) {
                 // copy compressed values over only
                 for (int k=1;k<=7;k++) {
                     if(combinations[j].contains("" + k) )
                     {
                        Region.copyRegionValues(arr1, tempArr, Region.region.getRegion(k), num_lvls);
                     }
                 }
              }

              // we now have the array ready to inverse transform
              // Calculate SSE
              // (1) Inverse Transform back
              for (int xferLevel2=loop_lvls; xferLevel2 >=1; xferLevel2/=2) {
                 HaarTransform.inverseTransform(arr1, arr2,
                                                dim1/xferLevel2,
                                                dim2/xferLevel2,
                                                dim3/xferLevel2);
                 ArrayUtil.copyArray(arr1, arr2);
              } // for xferLevel2

              // (2) Calc Error and Print results
              float sse = 0.0f;
              String tempStr = Region.generateLabel(combinations[j]);

              System.out.println("\n\n");
              System.out.println("\n\n SUM SQUARED ERROR REDUCTION FOR "
                             +tempStr );
              System.out.println("=============================");

              float error;
              String [] errorString = new String[1];
              errorString[0]="";

              error = ErrorUtility.calcAndPrintError( initArr, arr1,
                                                      tempStr, fileName,
                                                      range, log, true,
                                                      errorString);
           
              if ( error < lowest ) {
                  lowest = error;
                  lowest_index = j;
                  bestString = new String(errorString[0]);
              }
             
           }

           // Write the best one to results-best.csv
           String temp = combinations[lowest_index];
           try {
              log2.write(bestString);
           }
           catch(Exception e){
              System.out.println("unable to write to log2!\n");
              System.exit(0);
           }
        }
  
    }


    //////////////////////////////////////////////////////////////////////////
    // static boolean comboContainsDetail(String combo)
    //
    //
    //////////////////////////////////////////////////////////////////////////
    static boolean comboContainsDetail(String combo) {

        if (combo.contains("1") ||
            combo.contains("2") ||
            combo.contains("3") ||
            combo.contains("4") ||
            combo.contains("5") ||
            combo.contains("6") ||
            combo.contains("7") )
        {
           return true;
        }
        else
           return false;
    }

    /**************************************************
     *  generate_histogram
     *
     */
    static float[][] generate_histogram(float arr[][][], int num_bins, float IGNORE)
    {
       // Histogram the data - for help in better placement of bins
       Histogram hist = new Histogram(5000);
       // create histogram for summary only
       //hist.create_histogram(permArr, true, false );
       hist.create_histogram( arr, IGNORE );
       //hist.output_histogram(null);

       // The following call allows you to create a histogram of a single Detail region
       //hist.create_histogram_region(permArr, "SSD" );
       hist.output_histogram("histogram.csv");

       // optional - remap the data from histogram into specific number of bins
       float[][] map;
       int lock_count_per_side = 20;
       map = hist.generateMappedBins(num_bins, Histogram.mapping.MID, lock_count_per_side);

       System.out.println("\n\n\n===================================");
       for (int i=0;i<num_bins; i++) {
          System.out.println(i+": "+ map[i][0] + "\t\t" + map[i][1]);
       }


       // generate error per bin
       hist.printErrorPerBin(num_bins, arr, map, IGNORE);

       return map;
    }


    /**************************************************
     * testAllCombinations
     *
     *
     *
     *
     *
     *
     */
    public static void testAllCombinations( ){
       String [] arr = {"1","2","3","4","5","6","7"};
       String [] combinations;

       /* generate and test combinations for n=7, r=i*/
       for (int i=1;i<=1;i++) {
           combinations = CombinationGenerator.generateCombinations(arr,i,"0");

           // walk thru each combination
           for (int j=0;j<combinations.length;j++) {
              // walk through each posible region
              for (int k=1;k<=7;k++) {
                 // ZERO out all regions not in this combination!!
                 if ( !(combinations[j].contains(""+k)) ){

                 }

              }
           }
        }
    }

    //-------------------------------------------------------------------
    // compressCoefficients
    //
    // float arr1 - assumes every value in this array must be compressed
    //
    //--------------------------------------------------------------------
    private static void compressCoefficients(float arr1[][][],
                                       boolean compressSummary,
                                       boolean compressDetails,
                                       int num_lvls,
                                       int loop_lvls,
                                       int binCount,
                                       float summaryMap[][],
                                       float detailMap[][],
                                       String mapFilename)
    {
       int dim1=arr1.length;
       int dim2=arr1[0].length;
       int dim3=arr1[0][0].length;

       // now determine each bin value
       //SegmentedMapper dm = new SegmentedMapper(detailMap);

       //SegmentedMapper sm = new SegmentedMapper();
       //sm.mapCoefficients(tempArr, mapFilename);

       LinearMapper lm = new LinearMapper();
       lm.mapCoefficients(arr1, binCount);

       // just map all values
       float val;
       for (int i=0;i<dim1;i++) {     
          for (int j=0;j<dim2;j++) {
             for (int k=0;k<dim3;k++) {
                //tempArr[i][j][k] = dm.getMappedValue(tempArr[i][j][k]);
                arr1[i][j][k] = lm.getMappedValue(arr1[i][j][k]);
             }
          }
       }
    }


    //-------------------------------------------------------------------
    // calcAndPrintErrorNBest
    //
    //
    //
    //--------------------------------------------------------------------
    public static void calcAndPrintErrorNBest(  float  orig[][][],
                                                float  permArr[][][],
                                                String fileName,
                                                float range,
                                                BufferedWriter log,
                                                LinkedList<lvlRegionSSE> list,
                                                int num_lvls)
    {

        int loop_lvls =   (int)Math.pow(2,num_lvls-1);

        int dim1 = permArr.length;
        int dim2 = permArr[0].length;
        int dim3 = permArr[0][0].length;

        float [][][] arr1 = new float[dim1][dim2][dim3];
        float [][][] arr2 = new float[dim1][dim2][dim3];
        float [][][] arr3 = new float[dim1][dim2][dim3];

       // sort list of regions based on SSE - lowest to highest
       list = sortRegions(list, orig, permArr, loop_lvls);

       int loop_count = (int)Math.pow(8,num_lvls);

       // fill array with zeros
       ArrayUtil.setFloatArray(arr1, 0.0f);

       // set region values for SSS
       Region.copyRegionValues(permArr,
                               arr1,
                               Region.region.getRegion(0),
                               num_lvls);

       // iterate over each region to test
       ListIterator<lvlRegionSSE> itr = list.listIterator();
       lvlRegionSSE temp=null;
       int count = 0;

       // fill priority list with regions to keep
       while( itr.hasNext() )
       {
          temp = itr.next();

          // set region values for next region
          Region.copyRegionValues(permArr,
                                  arr1,
                                  Region.region.getRegion(temp.region),
                                  temp.lvl);

          // Calculate SSE
          // (1) Inverse Transform back

          // leave arr1 unchanged - copy into arr2
          ArrayUtil.copyArray(arr2, arr1);

          // (1) Inverse Transform back
          for (int xferLevel2=loop_lvls; xferLevel2 >=1; xferLevel2/=2) {
             HaarTransform.inverseTransform(arr2, arr3,
                                            dim1/xferLevel2,
                                            dim2/xferLevel2,
                                            dim3/xferLevel2);
             ArrayUtil.copyArray(arr2, arr3);
          } // for xferLevel2

          // (2) Calc Error and Print results
          float sse = 0.0f;

          System.out.println("\n\n");
          String tempStr=null;

          if (count == 0)
              tempStr = "SSS"+num_lvls;
          else
             tempStr= "SSS + BEST " + count + "(" + temp.getLabel() + ")" ;
          System.out.println("\n\n SUM SQUARED ERROR REDUCTION FOR SSS + BEST "
                             + count + "REGIONS" );
          System.out.println("=============================");

          float error[] = new float[1];


          ErrorUtility.calcAndPrintError( orig, arr2,tempStr, fileName, range,
                                          log, true, null);
          count++;
      } // while iiter hasNext()
    }


    //////////////////////////////////////////////////////////////
    // void sortRegions()
    //
    // args:
    //       LinkedList<lvlRegionSSE> list
    //
    //
    //
    //
    //////////////////////////////////////////////////////////////
    static LinkedList<lvlRegionSSE> sortRegions(LinkedList<lvlRegionSSE> list,
                                                float  orig[][][],
                                                float  permArr[][][],
                                                int loop_lvls)
    {
       int dim1, dim2, dim3;
       dim1 = orig.length;
       dim2 = orig[0].length;
       dim3 = orig[0][0].length;


       // array that keeps the current best regions
       float best[][][] = new float[dim1][dim2][dim3];
       ArrayUtil.setFloatArray(best, 0.0f);

       // new list
       LinkedList<lvlRegionSSE> list2= new LinkedList<lvlRegionSSE>();

//       //SSS(0), SSD(1), SDS(2),SDD(3), DSS(4), DSD(5), DDS(6), DDD(7);
//       lvlRegionSSE tempblah;
//       tempblah = new lvlRegionSSE(0,1);
//       list2.add(tempblah);
//       tempblah = new lvlRegionSSE(4,1);
//       list2.add(tempblah);
//       tempblah = new lvlRegionSSE(2,1);
//       list2.add(tempblah);
//       tempblah = new lvlRegionSSE(6,1);
//       list2.add(tempblah);
//       tempblah = new lvlRegionSSE(1,1);
//       list2.add(tempblah);
//       tempblah = new lvlRegionSSE(5,1);
//       list2.add(tempblah);
//       tempblah = new lvlRegionSSE(7,1);
//       list2.add(tempblah);
//       tempblah = new lvlRegionSSE(3,1);
//       list2.add(tempblah);
//
//       if (true)
//          return list2;


       // sort list of regions based on SSE - lowest to highest
       Collections.sort((List)list);   

       //---------------------------------------------------------
       // find SSS and place it first!
       ListIterator<lvlRegionSSE> itr = list.listIterator();

       lvlRegionSSE temp=null;
       lvlRegionSSE temp2=null;
       while( itr.hasNext() )
       {
          temp = itr.next();

          if ( temp.getLabel().startsWith("SSS")) {
             temp2 = temp;
             break;
          }
       }
       if (temp2 != null) {
          list.remove(temp2);
          list.addFirst(temp2);
       }

       //---------------------------------------------------------

       //---------------------------------------------------------
       // add first two from list to list2
       itr = list.listIterator();
       int count=1;
       while( itr.hasNext() && (count <= 2))
       {
          temp = itr.next();
          list2.add(temp);
          count++;

          // copy into array of best regions
          Region.copyRegionValues(best, permArr,
                                  Region.region.getRegion(temp.region),
                                  temp.lvl);
       }
       //---------------------------------------------------------

       //---------------------------------------------------------
       // Determine the next best to keep
       lvlRegionSSE keep=null;   
       float sse=Float.MAX_VALUE;

       // temp array for testing
       float tempArr[][][] = new float[dim1][dim2][dim3];
       float tempArr2[][][] = new float[dim1][dim2][dim3];
       ArrayUtil.setFloatArray(tempArr, 0.0f);
       ArrayUtil.setFloatArray(tempArr2, 0.0f);

      

       for (int i=0; i<6; i++) {


          // copy best regions into
          ArrayUtil.copyArray(tempArr, best);

          itr = list.listIterator();
          while( itr.hasNext() )
          {
             temp = itr.next();

             if ( !(list2.contains(temp)) ) {
              
                // 1) include region
                Region.copyRegionValues(tempArr, permArr,
                                     Region.region.getRegion(temp.region),
                                     temp.lvl);

                // 2) Reconstruct using current coefficients
                for (int xferLevel=loop_lvls; xferLevel >=1; xferLevel/=2) {
                   HaarTransform.inverseTransform(tempArr, tempArr2,
                                            dim1/xferLevel,
                                            dim2/xferLevel,
                                            dim3/xferLevel);
                   ArrayUtil.copyArray(tempArr, tempArr2);
                } // for xferLevel2

                // 3) Calculate SSE
                sse = calcAndPrintError(orig,
                                     tempArr,
                                     null, null,
                                     0, null,
                                     false, null);

                if ((keep == null) || (sse < keep.getSSE()) ){
                   keep = temp;//new lvlRegionSSE(temp.region, temp.lvl);                  
                }
               
             } // if ( !(list2.contains(temp)) )
          } // while (itr.hasNext())

          if (keep != null) {
             // add the best region
             keep.setSSE(sse);
             Region.copyRegionValues(best, permArr,
                                      Region.region.getRegion(keep.region),
                                      keep.lvl);
             list2.add(keep);
          }
          else {
             System.err.println(" keep == null! Aborting!!");
             System.exit(0);
          }
       }

       //---------------------------------------------------------

       return list2;
    }



//    //////////////////////////////////////////////////////////////
//    //
//    //////////////////////////////////////////////////////////////
//    static boolean listContains(LinkedList<tempClass> list, int i, int j, int k,
//                                int region)
//    {
//       ListIterator<tempClass> itr = list.listIterator();
//       tempClass temp;
//       while(itr.hasNext())
//       {
//          temp = itr.next();
//          if (temp.isEqual(i, j, k, region)) {
//              return true;
//           }
//       }
//       return false;
//    }

    //-------------------------------------------------------------------------
    // calcAndPrintError( float orig[][][], float changed[][][] )
    //
    // returns: ss error
    //-------------------------------------------------------------------------
    public static float calcAndPrintError(float  orig[][][],
                                          float  changed[][][],
                                          String descrip, String fileName,
                                          float range, BufferedWriter log,
                                          boolean print, String[] errorString)
    {
        if ((orig.length != changed.length) ||
            (orig[0].length != changed[0].length) ||
            (orig[0][0].length != changed[0][0].length))
        {
            System.out.println(" The two arrays aren't the same size! " +
                    "Aborting calculation and printing of error info.");
            System.exit(0);
        }

        int len1 = orig.length;
        int len2 = orig[0].length;
        int len3 = orig[0][0].length;

        float rel_error[][][] = new float[len1][len2][len3];
        float rms_error = 0.0f;
        float ss_error = 0.0f;
        float sum_abs_error = 0.0f;
        float max_abs_error = 0.0f;
        float avg_abs_error = 0.0f;

        float max = Float.MIN_VALUE;

        float temp=0.0f;

        for (int i = 0; i < len1; i++) {
            for (int j = 0; j < len2; j++) {
                for (int k = 0; k < len3; k++) {
                    temp = Math.abs(changed[i][j][k] - orig[i][j][k]);
                    // add to sum squared error
                    ss_error += Math.pow( temp, 2 );
                    sum_abs_error += temp;

                    if (temp > max_abs_error)
                        max_abs_error = temp;
                }
            }
        }
        int sz = (len1 * len2 * len3);
        avg_abs_error = (float)(sum_abs_error/sz);

        if (ss_error < 0) {
           System.out.println("FATAL ERROR: SS ERROR is negative ==>"
                              + ss_error);
           System.exit(1);
        }

        // // range was calculated when read in
        rms_error = (float)Math.sqrt(ss_error / sz);
        float normalized_rms = rms_error / range;

        if (print) {
           System.out.println("  sz = " + sz
                         + "  SSE = " + ss_error + "  rmse = " + rms_error
                         + "  normalized rmse = " + normalized_rms
                         + " sum ABS Error = " + sum_abs_error
                         + " avg ABS Error = " + avg_abs_error
                         + " max ABS Error" + max_abs_error);

           // Create output string to be written to file
           String line = fileName + "," + descrip + "," + ss_error + ","
               + rms_error + "," + normalized_rms + "," + range
               + "," + sum_abs_error
               + "," + max_abs_error
               + "," + avg_abs_error
               + "\n";


           if (errorString != null)
              errorString[0] += line;
           
           // write to output log
           try {
               System.out.println("Writing: " + line);
               log.write(line);
           } catch (Exception e) {
               e.printStackTrace();
               System.exit(0);
           }
        } // if (print)
        //return ss_error;
        return sum_abs_error;
    }





} // END OF CLASS


// THIS VERSION INCLUDES HISTOGRAM RELATED CALLS - COMMENTED OUT

//   //-------------------------------------------------------------------
//    // void calcAndPrintRegionErrors
//    //
//    // 1) Calculate the base error, which is the error if all regions are
//    //    set to 0
//    //    - output to log
//    //
//    // 2) Calculate the SSE and SSE_Reduction for each and every region
//    //    - output to log
//    //
//    // 3) Sort SSE_Reduction for all regions and calculate SSE for:
//    //     a) Best 1
//    //     b) Best 2
//    //     ...
//    //
//    //
//    //--------------------------------------------------------------------
//    public static void calcAndPrintRegionErrorsFloat_ALL_COMBINATIONS(
//                                          float [][][] initArr,
//                                          float ssError[],
//                                          float ssErrorReduct[],
//                                          float transformedArr[][][],
//                                          float compressedArr[][][],
//                                          int dim1, int dim2, int dim3,
//                                          String fileName,
//                                          float range, BufferedWriter log,
//                                          BufferedWriter log2,
//                                          float unh_region_error[][],
//                                          int num_lvls,
//                                          boolean compressSummary,
//                                          boolean compressDetails,
//                                          int num_bins,
//                                          String mapFilename
//                                         )
//    {
//       // conversions done in power of two
//       int loop_lvls =   (int)Math.pow(2,num_lvls-1);
//
//       // CREATE TEMPORARY ARRAYS
//       float [][][] arr1 = new float[dim1][dim2][dim3];
//       float [][][] arr2 = new float[dim1][dim2][dim3];
//
//       // array NOT to be modified - holds all coefficients
//       float [][][] permArr = new float[dim1][dim2][dim3];
//
//       // Array to be modified based on regions being kept
//       float [][][] tempArr = new float[dim1][dim2][dim3];
//
//       // copy initArr for transform
//       ArrayUtil.copyArray(arr1, initArr);
//
//       // each iteration is a transform
//       for (int xferLevel=1; xferLevel<=loop_lvls; xferLevel*=2) {
//          // perform transform
//          HaarTransform.transform(arr1, arr2, dim1/xferLevel,
//                                  dim2/xferLevel, dim3/xferLevel);
//          ArrayUtil.copyArray(arr1, arr2);
//       }
//       // save transformed array into permArr
//       ArrayUtil.copyArray(permArr, arr1);
//
//       ///////////////////////////////////////////////////////////////////
//       ///////////////////////////////////////////////////////////////////
//       //                                              ///////////////////
//       // Calculate Error Reduction per combination!!! ///////////////////
//       //                                              ///////////////////
//       ///////////////////////////////////////////////////////////////////
//       ///////////////////////////////////////////////////////////////////
//       int index =0;
//       int lvl=num_lvls;
//       int xferLevel=loop_lvls;
//       String bestString = null;
//
//       // number assigned per region SSS is region 0
//       String [] arr = {"0","1","2","3","4","5","6","7"};
//       String [] combinations;
//
//       // Generate histograms
//       float summaryMap[][]=null;
//       float detailMap[][]=null;
//
//       /* generate and test combinations for n=7, r=i*/
//       for (int i=1;i<=arr.length;i++) {
//           // generate combinations
//           combinations = CombinationGenerator.generateCombinations(arr,i,"");
//
//           try {
//              log.write("*** COMBINATIONS OF " + i + "***\n");
//           }catch(Exception e) {
//              System.out.println("Exception caught writing to log file." +
//                                    "Aborting!!");
//              System.exit(0);
//           }
//
//           float lowest= Float.MAX_VALUE;
//           int lowest_index=0;
//
//           // walk thru each combination
//           for (int j=0;j<combinations.length;j++) {
//
//               // initialize arr1 to zeros
//               ArrayUtil.setFloatArray(arr1, 0.0f);
//
//              // copy in regions we want to keep
//              for (int k=0; k<=7; k++) {
//                 // copy over regions in this combination!!
//                 if ( combinations[j].contains(""+k)){
//                    //Copy this regions values
//                    Region.copyRegionValues(arr1, permArr,
//                                            Region.region.getRegion(k), 1);
//                 }
//              }
//              String tempStr = Region.generateLabel(combinations[j]);
//
////                  // don't bother for just SSS - we don't compress it
////              if (combinations[j].equals("0") == false) {
//
//                 // copy initArr for transform
//                 ArrayUtil.copyArray(tempArr, arr1);
//
////                 for (int k=1;k<=7;k++) {
////                    if(combinations[j].contains("" + k) == false)
////                    {
////                       Region.setDetailRegionVal(tempArr,
////                                              Region.region.getRegion(k),
////                                              -9999.99f, 1);
////                    }
////                 }
//
//                 // set each value in SSS to -9999.99f, for histogram purposes
////                 Region.setDetailRegionVal(tempArr, Region.region.SSS, -9999.99f, 1);
//
//
//                 // compress regions we are keeping
//                 if (compressSummary || compressDetails) {
////                     System.out.println("==========================================================");
////                     System.out.println( "===  generating histogram for: " + tempStr );
////                     System.out.println("==========================================================");
////                     detailMap = generate_histogram(tempArr, num_bins, -9999.99f);
//
//                    // Byte Compression
//                    System.out.println(" compressing coefficients!");
//                    compressCoefficients(tempArr, compressSummary,  compressDetails,
//                               num_lvls, loop_lvls, num_bins,
//                               summaryMap,
//                               detailMap,
//                               mapFilename);
//                    System.out.println("Completed Compressing coefficients");
//                 }
//
//              //}
//
//              // copy compressed values over only
//              // - assumes no compression of summary
//              for (int k=1;k<=7;k++) {
//                 if(combinations[j].contains("" + k) )
//                 {
//                     Region.copyRegionValues(arr1, tempArr, Region.region.getRegion(k), 1);
//                 }
//              }
//
//              // we now have the array ready to inverse transform
//              // Calculate SSE
//              // (1) Inverse Transform back
//              for (int xferLevel2=loop_lvls; xferLevel2 >=1; xferLevel2/=2) {
//                 HaarTransform.inverseTransform(arr1, arr2,
//                                                dim1/xferLevel2,
//                                                dim2/xferLevel2,
//                                                dim3/xferLevel2);
//                 ArrayUtil.copyArray(arr1, arr2);
//              } // for xferLevel2
//
//              // (2) Calc Error and Print results
//              float sse = 0.0f;
//
//              System.out.println("\n\n");
//
//
//              System.out.println("\n\n SUM SQUARED ERROR REDUCTION FOR "
//                             +tempStr );
//              System.out.println("=============================");
//
//              float error;
//              String [] errorString = new String[1];
//              errorString[0]="";
//
//              error = ErrorUtility.calcAndPrintError( initArr, arr1,
//                                                      tempStr, fileName,
//                                                      range, log, true,
//                                                      errorString);
//              if (error < lowest) {
//                  lowest = error;
//                  lowest_index = j;
//                  bestString = new String(errorString[0]);
//              }
//           }
//
//           // Write the best one to results-best.csv
//           String temp = combinations[lowest_index];
//           try {
//              log2.write(bestString);
//           }
//           catch(Exception e){
//              System.out.println("unable to write to log2!\n");
//              System.exit(0);
//           }
//        }
//
//    }