//****************************************************************************
// FileUtil.java
//
// jimmy data = little endian
// head data  = big endian
//
// author: Jon Frain
//
//2-4-8-16-32-64-128-256-512
//3-6-12-24-48-96-192-384
//5-10-20-40-80-160-320-640
//7-14-28-56-112-224-448
//9-18-36-72-144-288-576
//11-22-44-88-176-352
//13-26-52-104-208-416
//15-30-60-120-240-480
//17-34-68-136-272-544
//**************************************************************************** 
import java.io.*;
import java.util.*;

public class FileUtil{
   private static float min;
   private static float max;
   private static float range;


   // CONSTRUCTOR
   public FileUtil( ) {
      max = range = min = 0.0f;
   }

   public float getMax(){
       return max;
   }

   public float getMin(){
       return min;
   }

   public float getRange(){
       return range;
   }

   //--------------------------------------------------------------------------
   // readDataFromFile(String filename)
   //
   // opens the specified file and reads doubles into a 3D array.
   //
   // caveats: This code is dumb and has hardcoded dimensions based on name 
   //          of the file being read.  As this is essentially unoptimized
   //          throw away test code I considered other things to be of higher
    //         priority.
   //--------------------------------------------------------------------------
    public double[][][] readFloatDataFromFileAsDouble(String filename,
                                               boolean bigEndian)
    {
       //int iSize = 296;
       //int jSize = 200;
       //int kSize = 608;

       int iSize = 112;
       int jSize = 112;
       int kSize = 384;

       if (filename.indexOf("head") > 0) 
       {
          iSize = 128;
          jSize = 256;
          kSize = 256;
       }

       double retArr[][][] = new double [iSize][jSize][kSize];
       float  tempFloat = 0;

       try {
          FileInputStream in = new FileInputStream(filename);
          DataInputStream din = new DataInputStream(in);
  
          float val=0.0f;
          
          max = Float.MIN_VALUE;
          min = Float.MAX_VALUE;  

          for (int i=0; i<iSize;i++) {
             for (int j=0; j<jSize;j++) {
                for (int k=0; k<kSize;k++) {               
                   tempFloat = din.readFloat();         

                   if (bigEndian == false)
                      tempFloat = byteSwap(tempFloat);

                   if (tempFloat > max)
                       max = tempFloat;

                   if (tempFloat < min)
                       min = tempFloat;

                   retArr[i][j][k] = (double)tempFloat;
                }
             }
          }  
          in.close();
       }
       catch(Exception e){
          System.out.println("Exception caught!");
          e.printStackTrace();
       }

       range = max - min;

       return retArr;
    }


   //--------------------------------------------------------------------------
   // readDataFromFile(String filename)
   //
   // opens the specified file and reads doubles into a 3D array.
   //
   // caveats: This code is dumb and has hardcoded dimensions based on name
   //          of the file being read.  As this is essentially unoptimized
   //          throw away test code I considered other things to be of higher
    //         priority.
   //--------------------------------------------------------------------------
    public float[][][] readFloatDataFromFile(String filename,
                                               boolean bigEndian)
    {
   
       int iSize = 112;
       int jSize = 112;
       int kSize = 384;

      
       
       if ( filename.contains("576x384x576")) {
          iSize = 576;
          jSize = 384;
          kSize = 576;
       }     
       else if ( filename.contains("576x192x288") ){
          iSize = 288;
          jSize = 192;
          kSize = 576;
       }
       else if ( filename.contains("630x200x300")){
          iSize = 300;
          jSize = 200;
          kSize = 630;
       }
       else if ( filename.contains("jimmy_384_partitions") ){
          iSize = 112/4;
          jSize = 112/4;
          kSize = 384/4;
       }
       else if ( filename.contains("head-float_partitions") ){
          iSize = 128/4;
          jSize = 256/4;
          kSize = 256/4;
       }
       else if(filename.contains ("head") )
       {
          iSize = 128;
          jSize = 256;
          kSize = 256;
       }
      else if(filename.contains ("raf") )
       {
          iSize = 256;
          jSize = 256;
          kSize = 512;
       }

       // - kludge for testing partitions - as they are all named the same
       //iSize = 288/8;
       //jSize = 192/8;
       //kSize = 576/8;

      


       float retArr[][][] = new float [iSize][jSize][kSize];
       float  tempFloat = 0;

       try {
          FileInputStream in = new FileInputStream(filename);
          DataInputStream din = new DataInputStream(in);

          float val=0.0f;

          max = Float.MIN_VALUE;
          min = Float.MAX_VALUE;

          for (int i=0; i<iSize;i++) {
             //System.out.println("  i="+i);
             for (int j=0; j<jSize;j++) {
                //System.out.println("     j="+j);
                for (int k=0; k<kSize;k++) {
                  
                   tempFloat = din.readFloat();

                   if (bigEndian == false)
                      tempFloat = byteSwap(tempFloat);

                   if (tempFloat > max)
                       max = tempFloat;

                   if (tempFloat < min)
                       min = tempFloat;

                   retArr[i][j][k] = tempFloat;
                }
             }
          }
          in.close();
       }
       catch(Exception e){
          System.out.println("Exception caught!");
          e.printStackTrace();
       }

       range = max - min;

       return retArr;
    }

   //--------------------------------------------------------------------------
   // byteSwap
   //
   // reads and returns one double as little endian
   //
   //--------------------------------------------------------------------------
   private float byteSwap(float val)
   {
       int intVal = Float.floatToIntBits ( val);

       int b1 = (intVal >>  0) & 0xff;
       int b2 = (intVal >>  8) & 0xff;
       int b3 = (intVal >> 16) & 0xff;
       int b4 = (intVal >> 24) & 0xff;
   
       int result = b1 << 24 | b2 << 16 | b3 << 8 | b4 << 0;
       return Float.intBitsToFloat(result);
   }


   //--------------------------------------------------------------------------
   // writeDataToFile(String filename, double arr[][][])
   //
   //
   //--------------------------------------------------------------------------
   public void writeDataToFile(String filename, float arr[][][],
                                      boolean bigEndian)
   {
       try {
          FileOutputStream out = new FileOutputStream(filename);
          DataOutputStream dos = new DataOutputStream(out);

          int lenDim1 = arr.length;
          int lenDim2 = arr[0].length;
          int lenDim3 = arr[0][0].length;

          for (int i=0; i<lenDim1;i++) {
             for (int j=0; j<lenDim2;j++) {
                for (int k=0; k<lenDim3;k++) {
                   if (bigEndian == false) {
                      arr[i][j][k] = byteSwap(arr[i][j][k]);
                   }
                   dos.writeFloat(arr[i][j][k]);
                }
             }
          }  
          out.close();
       }
       catch(Exception e){
          System.out.println("Exception caught!");
          e.printStackTrace();
       }
    }


   //--------------------------------------------------------------------------
   // writeStringToFile(String filename, String line)
   //
   //
   //--------------------------------------------------------------------------
   public void writeStringToFile(String filename, String s)
   {
       try {
          FileOutputStream out = new FileOutputStream(filename);

          for (int i=0; i<s.length();i++){
             out.write((byte)s.charAt(i));
          }
          out.flush();
          out.close();
       }
       catch(Exception e){
          System.out.println("Exception caught!");
          e.printStackTrace();
       }
    }


 

   //--------------------------------------------------------------------------
   // convert( int i1, int j1, int k1, int i2, int j2, int k2,
   //                      String filename, String outFilename)
   //
   // - reduces the dimensions of a file into a new file
    //- simply truncates the first elements in each dimension
   //
   //--------------------------------------------------------------------------
    public void convert( int i1, int j1, int k1, int i2, int j2, int k2,
                         String filename, String outFilename)
    {
       int iSize = i1;
       int jSize = j1;
       int kSize = k1;

       int iStart = i1 - i2;
       int jStart = j1 - j2;
       int kStart = k1 - k2;

       float retArr[][][] = new float [iSize-iStart][jSize-jStart][kSize-kStart];
       float  tempFloat = 0;


       try {
          FileInputStream in = new FileInputStream(filename);
          DataInputStream din = new DataInputStream(in);

          for (int i=0; i<iSize; i++) {
              System.out.println("i="+i);
             for (int j=0; j<jSize; j++) {
                for (int k=0; k<kSize; k++) {
                   tempFloat = din.readFloat();

                    if ( (k >= kStart) && (j >= jStart) && (i >= iStart)) {
                          retArr[i-iStart][j-jStart][k-kStart] = tempFloat;
                    }
                }
             }
          }
          in.close();
       }
       catch(Exception e){
          System.out.println("Exception caught!");
          e.printStackTrace();
          System.exit(0);
       }
       // write data to file
       writeDataToFile(outFilename, retArr, true);

    }


   //--------------------------------------------------------------------
   // main
   //
   //-------------------------------------------------------------------- 
    public static void main(String [ ] args) {
   
   }      
}



