//****************************************************************************
// TestBench2.java
//
// 
//
//  Note:
//
//    1) Code assumes even sized dimensions of arrays when calculating
//       half the size of a dimension
//
//    2) Code is most likely inefficient in that it probably creates too
//       many temporary arrays to perform a transform or inverse transform
//
// author: Jon Frain
//
//**************************************************************************** 
import java.io.*;
import java.util.*;

public class HaarTransform {
   
   // CONSTRUCTOR
   public HaarTransform( ) {

   }

   //--------------------------------------------------------------------------
   //  void transform()
   //
   //  args:
   //   double initArr[][][] - array to be transformed
   //   double transformedArr[][][] - array to place result into
   //   int dim1, int dim2, int dim3 -
   //       dimensions of array to be transformed arr[dim1][dim2][dim3]
   //
   //  Purpose:
   //     Performs one level of transformation, ie 4x4x4 to 2x2x2
   //
   //--------------------------------------------------------------------------
   public static void transform( double initArr[][][],
                                 double transformedArr[][][],
                                 int dim1, int dim2, int dim3 )
   {
       // lengths needed for storing entire array - not just
      //   portion to be transformed
       int lenDim1 = initArr.length;
       int lenDim2 = initArr[0].length;
       int lenDim3 = initArr[0][0].length;

       // lengths used for portion being transformed
       int halfDim1 = dim1/2;
       int halfDim2 = dim2/2;
       int halfDim3 = dim3/2;

       double retArr[][][] = new double[lenDim1][lenDim2][lenDim3];

       // transform across cols (k)
       for (int i=0; i<dim1; i++) {
          for (int j=0; j<dim2; j++) {
             for (int k=0; k<dim3; k+=2) {
                retArr[i][j][k/2]          = initArr[i][j][k]*0.5f
                                             + initArr[i][j][k+1]*0.5f;
                retArr[i][j][k/2+halfDim3] = retArr[i][j][k/2]
                                             - initArr[i][j][k];
             }
          }
       }

       double retArr2[][][] = new double[lenDim1][lenDim2][lenDim3];
       // transform across rows (j)
       for (int i=0; i<dim1; i++) {
          for (int j=0; j<dim2; j+=2) {
             for (int k=0; k<dim3; k++) {
                retArr2[i][j/2][k]          = retArr[i][j][k]*0.5f
                                              + retArr[i][j+1][k]*0.5f;
                retArr2[i][j/2+halfDim2][k] = retArr2[i][j/2][k]
                                              - retArr[i][j][k];
             }
          }
       }

       ArrayUtil.copyArray(transformedArr, initArr);
       // transform across slices (i)
       for (int i=0; i<dim1; i+=2) {
          for (int j=0; j<dim2; j++) {
             for (int k=0; k<dim3; k++) {
                transformedArr[i/2][j][k]          = retArr2[i][j][k]*0.5f
                                              + retArr2[i+1][j][k]*0.5f;
                transformedArr[i/2+halfDim1][j][k] = transformedArr[i/2][j][k]
                                              - retArr2[i][j][k];
             }
          }
       }
   }


   //--------------------------------------------------------------------------
   //  void transform()
   //
   //  args:
   //   double initArr[][][] - array to be transformed
   //   double transformedArr[][][] - array to place result into
   //   int dim1, int dim2, int dim3 -
   //       dimensions of array to be transformed arr[dim1][dim2][dim3]
   //
   //  Purpose:
   //     Performs one level of transformation, ie 4x4x4 to 2x2x2
   //
   //--------------------------------------------------------------------------
   public static void transform( float initArr[][][],
                                 float transformedArr[][][],
                                 int dim1, int dim2, int dim3 )
   {
       // lengths needed for storing entire array - not just
      //   portion to be transformed
       int lenDim1 = initArr.length;
       int lenDim2 = initArr[0].length;
       int lenDim3 = initArr[0][0].length;

       // lengths used for portion being transformed
       int halfDim1 = dim1/2;
       int halfDim2 = dim2/2;
       int halfDim3 = dim3/2;

       float retArr[][][] = new float[lenDim1][lenDim2][lenDim3];

       // transform across cols (k)
       for (int i=0; i<dim1; i++) {
          for (int j=0; j<dim2; j++) {
             for (int k=0; k<dim3; k+=2) {
                retArr[i][j][k/2]          = initArr[i][j][k]*0.5f
                                             + initArr[i][j][k+1]*0.5f;
                retArr[i][j][k/2+halfDim3] = retArr[i][j][k/2]
                                             - initArr[i][j][k];
             }
          }
       }

       float retArr2[][][] = new float[lenDim1][lenDim2][lenDim3];
       // transform across rows (j)
       for (int i=0; i<dim1; i++) {
          for (int j=0; j<dim2; j+=2) {
             for (int k=0; k<dim3; k++) {
                retArr2[i][j/2][k]          = retArr[i][j][k]*0.5f
                                              + retArr[i][j+1][k]*0.5f;
                retArr2[i][j/2+halfDim2][k] = retArr2[i][j/2][k]
                                              - retArr[i][j][k];
             }
          }
       }

       ArrayUtil.copyArray(transformedArr, initArr);
       // transform across slices (i)
       for (int i=0; i<dim1; i+=2) {
          for (int j=0; j<dim2; j++) {
             for (int k=0; k<dim3; k++) {
                transformedArr[i/2][j][k]          = retArr2[i][j][k]*0.5f
                                              + retArr2[i+1][j][k]*0.5f;
                transformedArr[i/2+halfDim1][j][k] = transformedArr[i/2][j][k]
                                              - retArr2[i][j][k];
             }
          }
       }
   }


   //--------------------------------------------------------------------------
   //  void transform()
   //
   //  args:
   //   double initArr[][][] - array to be transformed
   //
   //   double transformedArr[][][] - array to place result into
   //
   //   int start1, start2, start3 -
   //
   //   int dim1, int dim2, int dim3 -
   //       dimensions of array to be transformed arr[dim1][dim2][dim3]
   //
   //  Purpose:
   //     Performs one level of transformation, ie 4x4x4 to 2x2x2
   //     - this version allows the starting location to be non (0,0,0)
   //
   //--------------------------------------------------------------------------
   public static void transform( double initArr[][][],
                                 double transformedArr[][][],
                                 int start1, int start2, int start3,
                                 int dim1, int dim2, int dim3 )
   {
       // lengths needed for storing entire array - not just
       //   portion to be transformed
       int lenDim1 = initArr.length;
       int lenDim2 = initArr[0].length;
       int lenDim3 = initArr[0][0].length;

       // lengths used for portion being transformed
       int halfDim1 = dim1/2;
       int halfDim2 = dim2/2;
       int halfDim3 = dim3/2;

       double retArr[][][] = new double[lenDim1][lenDim2][lenDim3];
       // array created for performing the transform
       double subArr[][][] = new double[dim1][dim2][dim3];

       // transform across cols (k)
       for (int i=0; i<dim1; i++) {
          for (int j=0; j<dim2; j++) {
             for (int k=0; k<dim3; k+=2) {
                subArr[i][j][k/2]          = initArr[start1+i][start2+j][start3+k]*0.5f
                                             + initArr[start1+i][start2+j][start3+k+1]*0.5f;
                subArr[i][j][k/2+halfDim3] = subArr[i][j][k/2]
                                             - initArr[start1+i][start2+j][start3+k];
             }
          }
       }

//       System.out.println("################ subArr ####################");
//       ArrayUtil.printArray(subArr, dim1, dim2, dim3);

       double subArr2[][][] = new double[dim1][dim2][dim3];
       // transform across rows (j)
       for (int i=0; i<dim1; i++) {
          for (int j=0; j<dim2; j+=2) {
             for (int k=0; k<dim3; k++) {
                subArr2[i][j/2][k]          = subArr[i][j][k]*0.5f
                                              + subArr[i][j+1][k]*0.5f;
                subArr2[i][j/2+halfDim2][k] = subArr2[i][j/2][k]
                                              - subArr[i][j][k];
             }
          }
       }

//       System.out.println("################ subArr2 ####################");
//       ArrayUtil.printArray(subArr2, dim1, dim2, dim3);

       // transform across slices (i)
       for (int i=0; i<dim1; i+=2) {
          for (int j=0; j<dim2; j++) {
             for (int k=0; k<dim3; k++) {
                transformedArr[start1+i/2][start2+j][start3+k] = 
                                subArr2[i][j][k]*0.5f + subArr2[i+1][j][k]*0.5f;
                transformedArr[start1+i/2+halfDim1][start2+j][start3+k] = 
                                transformedArr[start1+i/2][start2+j][start3+k]
                                - subArr2[i][j][k];
             }
          }
       }

   }


   //--------------------------------------------------------------------------
   //  void transform()
   //
   //  args:
   //   double initArr[][][] - array to be transformed
   //
   //   double transformedArr[][][] - array to place result into
   //
   //   int start1, start2, start3 -
   //
   //   int dim1, int dim2, int dim3 -
   //       dimensions of array to be transformed arr[dim1][dim2][dim3]
   //
   //  Purpose:
   //     Performs one level of transformation, ie 4x4x4 to 2x2x2
   //     - this version allows the starting location to be non (0,0,0)
   //
   //--------------------------------------------------------------------------
   public static void transform( float initArr[][][],
                                 float transformedArr[][][],
                                 int start1, int start2, int start3,
                                 int dim1, int dim2, int dim3 )
   {
       // lengths needed for storing entire array - not just
       //   portion to be transformed
       int lenDim1 = initArr.length;
       int lenDim2 = initArr[0].length;
       int lenDim3 = initArr[0][0].length;

       // lengths used for portion being transformed
       int halfDim1 = dim1/2;
       int halfDim2 = dim2/2;
       int halfDim3 = dim3/2;

       float retArr[][][] = new float[lenDim1][lenDim2][lenDim3];
       // array created for performing the transform
       float subArr[][][] = new float[dim1][dim2][dim3];

       // transform across cols (k)
       for (int i=0; i<dim1; i++) {
          for (int j=0; j<dim2; j++) {
             for (int k=0; k<dim3; k+=2) {
                subArr[i][j][k/2] = initArr[start1+i][start2+j][start3+k]*0.5f
                                  + initArr[start1+i][start2+j][start3+k+1]*0.5f;
                subArr[i][j][k/2+halfDim3]
                                  = subArr[i][j][k/2]
                                  - initArr[start1+i][start2+j][start3+k];
             }
          }
       }

//       System.out.println("################ subArr ####################");
//       ArrayUtil.printArray(subArr, dim1, dim2, dim3);

       float subArr2[][][] = new float[dim1][dim2][dim3];
       // transform across rows (j)
       for (int i=0; i<dim1; i++) {
          for (int j=0; j<dim2; j+=2) {
             for (int k=0; k<dim3; k++) {
                subArr2[i][j/2][k]          = subArr[i][j][k]*0.5f
                                              + subArr[i][j+1][k]*0.5f;
                subArr2[i][j/2+halfDim2][k] = subArr2[i][j/2][k]
                                              - subArr[i][j][k];
             }
          }
       }

//       System.out.println("################ subArr2 ####################");
//       ArrayUtil.printArray(subArr2, dim1, dim2, dim3);

       // transform across slices (i)
       for (int i=0; i<dim1; i+=2) {
          for (int j=0; j<dim2; j++) {
             for (int k=0; k<dim3; k++) {
                transformedArr[start1+i/2][start2+j][start3+k] =
                                subArr2[i][j][k]*0.5f + subArr2[i+1][j][k]*0.5f;
                transformedArr[start1+i/2+halfDim1][start2+j][start3+k] =
                                transformedArr[start1+i/2][start2+j][start3+k]
                                - subArr2[i][j][k];
             }
          }
       }

   }

   //--------------------------------------------------------------------------
   // void inverseTransform (  )
   //
   // args:
   //   double initArr[][][] - array to be inverse transformed
   //   double transformedArr[][][] - array to place result into
   //   int dim1, int dim2, int dim3 -
   //       dimensions of resultant array after inverse transformation
   //           arr[dim1][dim2][dim3]
   //
   //  Purpose:
   //     Performs one level of inverse transformation, ie 2x2x2 to 4x4x4
   //--------------------------------------------------------------------------
   public static void inverseTransform( double initArr[][][],
                                        double transformedArr[][][],
                                        int dim1, int dim2, int dim3)
   {
       int lenDim1 = initArr.length;
       int lenDim2 = initArr[0].length;
       int lenDim3 = initArr[0][0].length;

       int halfDim1 = dim1/2;
       int halfDim2 = dim2/2;
       int halfDim3 = dim3/2;

       double retArr[][][] = new double[lenDim1][lenDim2][lenDim3];

       for (int i=0; i<dim1; i+=2) {
          for (int j=0; j<dim2; j++) {
             for (int k=0; k<dim3; k++) {
                retArr[i][j][k]   = initArr[i/2][j][k]
                                    - initArr[i/2+halfDim1][j][k];
                retArr[i+1][j][k] = initArr[i/2][j][k]
                                    + initArr[i/2+halfDim1][j][k];
             }
          }
       }

       double retArr2[][][] = new double [lenDim1][lenDim2][lenDim3];
       for (int i=0; i<dim1; i++) {
          for (int j=0; j<dim2; j+=2) {
             for (int k=0; k<dim3; k++) {
                retArr2[i][j][k]   = retArr[i][j/2][k]
                                     - retArr[i][j/2+halfDim2][k];
                retArr2[i][j+1][k] = retArr[i][j/2][k]
                                     + retArr[i][j/2+halfDim2][k];
             }
          }
       }

       
       ArrayUtil.copyArray(transformedArr, initArr);
       for (int i=0; i<dim1; i++) {
          for (int j=0; j<dim2; j++) {
             for (int k=0; k<dim3; k+=2) {
                transformedArr[i][j][k]   = 
                               retArr2[i][j][k/2] - retArr2[i][j][k/2+halfDim3];
                transformedArr[i][j][k+1] = 
                               retArr2[i][j][k/2] + retArr2[i][j][k/2+halfDim3];
             }
          }
       }
   }


   //--------------------------------------------------------------------------
   // void inverseTransform (  )
   //
   // args:
   //   double initArr[][][] - array to be inverse transformed
   //   double transformedArr[][][] - array to place result into
   //   int dim1, int dim2, int dim3 -
   //       dimensions of resultant array after inverse transformation
   //           arr[dim1][dim2][dim3]
   //
   //  Purpose:
   //     Performs one level of inverse transformation, ie 2x2x2 to 4x4x4
   //--------------------------------------------------------------------------
   public static void inverseTransform( float initArr[][][],
                                        float transformedArr[][][],
                                        int dim1, int dim2, int dim3)
   {

      // System.out.println("--- inverse xform -- dim1="+dim1+"  dim2="+dim2 +
      //                    "  dim3=" + dim3);

       int lenDim1 = initArr.length;
       int lenDim2 = initArr[0].length;
       int lenDim3 = initArr[0][0].length;

       int halfDim1 = dim1/2;
       int halfDim2 = dim2/2;
       int halfDim3 = dim3/2;

       float retArr[][][] = new float[lenDim1][lenDim2][lenDim3];

       for (int i=0; i<dim1; i+=2) {
          for (int j=0; j<dim2; j++) {
             for (int k=0; k<dim3; k++) {
                retArr[i][j][k]   = initArr[i/2][j][k]
                                    - initArr[i/2+halfDim1][j][k];
                retArr[i+1][j][k] = initArr[i/2][j][k]
                                    + initArr[i/2+halfDim1][j][k];
             }
          }
       }

       float retArr2[][][] = new float [lenDim1][lenDim2][lenDim3];
       for (int i=0; i<dim1; i++) {
          for (int j=0; j<dim2; j+=2) {
             for (int k=0; k<dim3; k++) {
                retArr2[i][j][k]   = retArr[i][j/2][k]
                                     - retArr[i][j/2+halfDim2][k];
                retArr2[i][j+1][k] = retArr[i][j/2][k]
                                     + retArr[i][j/2+halfDim2][k];
             }
          }
       }


       ArrayUtil.copyArray(transformedArr, initArr);
       for (int i=0; i<dim1; i++) {
          for (int j=0; j<dim2; j++) {
             for (int k=0; k<dim3; k+=2) {
                transformedArr[i][j][k]   =
                               retArr2[i][j][k/2] - retArr2[i][j][k/2+halfDim3];
                transformedArr[i][j][k+1] =
                               retArr2[i][j][k/2] + retArr2[i][j][k/2+halfDim3];
             }
          }
       }
   }

   //--------------------------------------------------------------------------
   //  void inverseTransform()
   //
   //  args:
   //   double initArr[][][] - array to be transformed
   //
   //   double transformedArr[][][] - array to place result into
   //
   //   int start1, start2, start3 -
   //
   //   int dim1, int dim2, int dim3 -
   //       dimensions of array to be transformed arr[dim1][dim2][dim3]
   //
   //  Purpose:
   //     Performs one level of transformation, ie 4x4x4 to 2x2x2
   //     - this version allows the starting location to be non (0,0,0)
   //
   //--------------------------------------------------------------------------
   public static void inverseTransform( double initArr[][][],
                                 double transformedArr[][][],
                                 int start1, int start2, int start3,
                                 int dim1, int dim2, int dim3 )
   {

       int halfDim1 = dim1/2;
       int halfDim2 = dim2/2;
       int halfDim3 = dim3/2;

       double subArr[][][] = new double [dim1][dim2][dim3];

       for (int i=0; i<dim1; i+=2) {
          for (int j=0; j<dim2; j++) {
             for (int k=0; k<dim3; k++) {
                subArr[i][j][k]   = initArr[start1 + i/2][start2 + j][ start3 + k]
                                  - initArr[start1 + i/2 + halfDim1][start2 + j][start3 + k];
                subArr[i+1][j][k] = initArr[start1 + i/2][start2 + j][start3 + k]
                                  + initArr[start1 + i/2 + halfDim1][start2 + j][start3 + k];
             }
          }
       }

       double subArr2[][][] = new double [dim1][dim2][dim3];
       for (int i=0; i<dim1; i++) {
          for (int j=0; j<dim2; j+=2) {
             for (int k=0; k<dim3; k++) {
                subArr2[i][j][k]   = subArr[i][j/2][k]
                                     - subArr[i][j/2+halfDim2][k];
                subArr2[i][j+1][k] = subArr[i][j/2][k]
                                     + subArr[i][j/2+halfDim2][k];
             }
          }
       }

       for (int i=0; i<dim1; i++) {
          for (int j=0; j<dim2; j++) {
             for (int k=0; k<dim3; k+=2) {
                transformedArr[start1+i][start2+j][start3+k]   = 
                               subArr2[i][j][k/2] - subArr2[i][j][k/2+halfDim3];
                transformedArr[start1+i][start2+j][start3+k+1] = 
                               subArr2[i][j][k/2] + subArr2[i][j][k/2+halfDim3];
             }
          }
       }            
   }


   //--------------------------------------------------------------------------
   //  void inverseTransform()
   //
   //  args:
   //   float initArr[][][] - array to be transformed
   //
   //   float transformedArr[][][] - array to place result into
   //
   //   int start1, start2, start3 -
   //
   //   int dim1, int dim2, int dim3 -
   //       dimensions of array to be transformed arr[dim1][dim2][dim3]
   //
   //  Purpose:
   //     Performs one level of transformation, ie 4x4x4 to 2x2x2
   //     - this version allows the starting location to be non (0,0,0)
   //
   //--------------------------------------------------------------------------
   public static void inverseTransform( float initArr[][][],
                                 float transformedArr[][][],
                                 int start1, int start2, int start3,
                                 int dim1, int dim2, int dim3 )
   {

       int halfDim1 = dim1/2;
       int halfDim2 = dim2/2;
       int halfDim3 = dim3/2;

       float subArr[][][] = new float [dim1][dim2][dim3];

       for (int i=0; i<dim1; i+=2) {
          for (int j=0; j<dim2; j++) {
             for (int k=0; k<dim3; k++) {
                subArr[i][j][k]   = initArr[start1 + i/2][start2 + j][ start3 + k]
                                    - initArr[start1 + i/2 + halfDim1][start2 + j][start3 + k];
                subArr[i+1][j][k] = initArr[start1 + i/2][start2 + j][start3 + k]
                                    + initArr[start1 + i/2 + halfDim1][start2 + j][start3 + k];
             }
          }
       }

       float subArr2[][][] = new float [dim1][dim2][dim3];
       for (int i=0; i<dim1; i++) {
          for (int j=0; j<dim2; j+=2) {
             for (int k=0; k<dim3; k++) {
                subArr2[i][j][k]   = subArr[i][j/2][k]
                                     - subArr[i][j/2+halfDim2][k];
                subArr2[i][j+1][k] = subArr[i][j/2][k]
                                     + subArr[i][j/2+halfDim2][k];
             }
          }
       }

       for (int i=0; i<dim1; i++) {
          for (int j=0; j<dim2; j++) {
             for (int k=0; k<dim3; k+=2) {
                transformedArr[start1+i][start2+j][start3+k]   =
                               subArr2[i][j][k/2] - subArr2[i][j][k/2+halfDim3];
                transformedArr[start1+i][start2+j][start3+k+1] =
                               subArr2[i][j][k/2] + subArr2[i][j][k/2+halfDim3];
             }
          }
       }
   }

   //--------------------------------------------------------------------------
   //  void transform()
   //
   //  args:
   //   double initArr[] - array to be transformed
   //   double transformedArr[] - array to place result into
   //   int dim1 - dimension of array to be transformed arr[dim1]
   //
   //  Purpose:
   //     Performs one level of transformation, ie 16 down to 8
   //
   //--------------------------------------------------------------------------
   public static void transform( double initArr[], double transformedArr[],
                                   int dim1 )
   {
       int halfDim1 = dim1/2;

       // transform across slices (i)
       for (int i=0; i<dim1; i+=2) {
                transformedArr[i/2] = initArr[i]*0.5f + initArr[i+1]*0.5f;
                transformedArr[i/2+halfDim1] = transformedArr[i/2] - initArr[i];
       }
   }

   //--------------------------------------------------------------------------
   // void inverseTransform (  )
   //
   // args:
   //   double initArr[][][] - array to be inverse transformed
   //   double transformedArr[][][] - array to place result into
   //   int dim1, int dim2, int dim3 -
   //       dimensions of resultant array after inverse transformation
   //           arr[dim1][dim2][dim3]
   //
   //  Purpose:
   //     Performs one level of inverse transformation, ie 2x2x2 to 4x4x4
   //--------------------------------------------------------------------------
   public static void inverseTransform( double initArr[],
                                          double transformedArr[],
                                          int dim1)
   {
       int halfDim1 = dim1/2;

       for (int i=0; i<dim1; i+=2) {
          transformedArr[i]   = initArr[i/2] - initArr[i/2+halfDim1];
          transformedArr[i+1] = initArr[i/2] + initArr[i/2+halfDim1];
       }
   }

   //--------------------------------------------------------------------------
   // void testNormalWavelets (  )
   //
   // args:
   //
   //  Purpose:
   //     Tests basic functionality
   //
   //          ---------------------
   //          | SS | SD |         |
   //          |---------|    SD   |
   //          | DS | DD |         |
   //          ---------------------
   //          |         |         |
   //          |    DS   |    DD   |
   //          |         |         |
   //          ---------------------
   //--------------------------------------------------------------------------
   public void testNormalWavelets(){
      
       // create a 4x4x4 array
       double[][][] initArr = new double [4][4][4];

       // fill array with incrementing doubles beginning at 1.0
       double val=1.0f;
       for (int i=0; i<4;i++){
          for (int j=0; j<4;j++){
             for (int k=0; k<4;k++){
                initArr[i][j][k] = val++;
             }
          }
       }

       // print starting array
       ArrayUtil.printArray(initArr);

       double[][][] arr2 = new double [4][4][4];
       double[][][] arr3 = new double [4][4][4];
       double[][][] arr4 = new double [4][4][4];
       double[][][] arr5 = new double [4][4][4];

       // perform single transformation and print
       transform(initArr, arr2, 4, 4, 4);
       ArrayUtil.printArray(arr2);

       // perform second transformation and print
       transform(arr2, arr3, 2, 2, 2);
       ArrayUtil.printArray(arr3);

       // perform single inverse transformation and print
       inverseTransform(arr3, arr4, 2, 2,2);
       ArrayUtil.printArray(arr4);

       // perform second inverse transformation and print
       // - should be starting array
       inverseTransform(arr4, arr5, 4, 4, 4);
       ArrayUtil.printArray(arr5);

   }


   //--------------------------------------------------------------------------
   // void testRecursiveWavelets (  )
   //
   // args:
   //
   // Purpose:
   //     Tests modified wavelets
   //
   //          ---------------------
   //          | SS | SD | SS | SD |
   //          |---------|---------|
   //          | DS | DD | DS | DD |
   //          ---------------------
   //          | SS | SD | SS | SD |
   //          |---------|---------|
   //          | DS | DD | DS | DD |
   //          ---------------------
   //
   //
   //--------------------------------------------------------------------------
   public void testRecursiveWavelets(){
       int DIM = 8;

       // create a 4x4x4 array
       double[][][] initArr = new double [DIM][DIM][DIM];

       // fill array with incrementing doubles beginning at 1.0
       double val=1.0f;
       for (int i=0; i<DIM;i++){
          for (int j=0; j<DIM;j++){
             for (int k=0; k<DIM;k++){
                initArr[i][j][k] = val++;
             }
          }
       }

       // print starting array
       ArrayUtil.printArray(initArr);

       double[][][] arr1 = new double [DIM][DIM][DIM];
       double[][][] arr2 = new double [DIM][DIM][DIM];

       // Perform Transforms
       int increment;
       ArrayUtil.copyArray(arr1, initArr);
       for (int xferLevel=1; xferLevel <=4; xferLevel*=2) {
          increment = DIM/xferLevel;
          for (int i=0;i<DIM;i+=increment) {
              for (int j=0;j<DIM;j+=increment) {
                 for (int k=0;k<DIM;k+=increment) {
//                     System.out.println("*** transforming:");
//                     System.out.println("   - start = " + i + "," + j + "," + k);
//                     System.out.println("   - dim = " + increment);
                    transform(arr1, arr2, i, j, k, increment, increment, increment);
                 }
              }
          }
          ArrayUtil.copyArray(arr1, arr2);
       }

       // debug print
       ArrayUtil.printArray(arr2);

       // Inverse Transform back
       for (int xferLevel=4; xferLevel >=1; xferLevel/=2) {
          increment = DIM/xferLevel;
          for (int i=0;i<DIM;i+=increment) {
              for (int j=0;j<DIM;j+=increment) {
                 for (int k=0;k<DIM;k+=increment) {
//                    System.out.println("*** inverse transforming:");
//                    System.out.println("   - start = " + i + "," + j + "," + k);
//                    System.out.println("   - dim = " + increment);
                    inverseTransform(arr1, arr2, i, j, k, increment, increment, increment);
                 }
              }
          }
          ArrayUtil.copyArray(arr1, arr2);
       }

       ArrayUtil.printArray(arr2);
       
   }

   //--------------------------------------------------------------------
   // main
   //
   //-------------------------------------------------------------------- 
    public static void main(String [ ] args) {
       HaarTransform ht = new HaarTransform();
       
       //ht.testNormalWavelets();

       ht.testRecursiveWavelets();
    }
}



