/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

/**
 * Histogram.java
 * 
 * 
 * Creates a histogram of the summary and/or detail coefficients for a 
 * single level of transformed data.
 * 
 * The purpose is to try and determine how to best create bins to compress these
 * coefficients
 *
 * @author Jon
 *
 * 2/28/11
 *
 */
import java.io.*;
import java.util.*;


public class Histogram {

    public enum mapping {MID, MEDIAN};
    private int counts[];
    private int num_bins;
    private int numVals;
    private float min, max;
    private float binSpacing;

    Histogram(int bin_count){
        num_bins = bin_count;
        counts = new int[num_bins];

        reset_histogram();
    }

    void reset_histogram(){
        min = Float.MAX_VALUE;
        max = Float.MIN_VALUE;
        
        for (int i=0; i<num_bins; i++) {
             counts[i]=0;
        }

    }

    ///////////////////////////////////////////////////////////////////////
    //
    // create_histogram
    //
    //
    //
    ///////////////////////////////////////////////////////////////////////
    void create_histogram( float data[][][], float IGNORE )
    {
        int dim1 = data.length;
        int dim2 = data[0].length;
        int dim3 = data[0][0].length;
    
        numVals = 0;
        int index;

        // find min and max values
        get_min_max(data, IGNORE);
          
        for (int i=0; i<dim1; i++) {
            for (int j=0; j<dim2; j++) {
                for (int k=0; k<dim3; k++) {
                    if ( data[i][j][k] != IGNORE ) {
                       numVals++;
                       index = (int)( (data[i][j][k]-min)/binSpacing );
                       if (index >= num_bins)
                          index = num_bins-1;
                       counts[index]++;
                    }
                }             
            }           
        }
     
    }

//    ///////////////////////////////////////////////////////////////////
//    // create_histogram_region
//    //
//    ///////////////////////////////////////////////////////////////////
//    void create_histogram_region(float data[][][], String region){
//
//       if (region == null) {
//          System.out.println("ERROR: histogram_detail_region() arg region is null ");
//          System.exit(1);
//       }
//
//       // determine the size in each dimension
//       int lenDim1 = data.length;
//       int lenDim2 = data[0].length;
//       int lenDim3 = data[0][0].length;
//
//       int halfDim1 = (lenDim1/2);
//       int halfDim2 = (lenDim2/2);
//       int halfDim3 = (lenDim3/2);
//
//       int i1, i2; // slice
//       int j1, j2; // row
//       int k1, k2; // col
//       i1=i2=j1=j2=k1=k2=0;
//
//       if (region.equals("SSD")) {
//              i1 = 0;
//              i2 = halfDim1;
//
//              j1 = 0;
//              j2 = halfDim2;
//
//              k1 = halfDim3;
//              k2 = lenDim3;
//       }
//       else if(region.equals("SDS")) {
//              i1 = 0;
//              i2 = halfDim1;
//
//              j1 = halfDim2;
//              j2 = lenDim2;
//
//              k1 = 0;
//              k2 = halfDim3;
//        }
//        else if(region.equals("SDD")) {
//              i1 = 0;
//              i2 = halfDim1;
//
//              j1 = halfDim2;
//              j2 = lenDim2;
//
//              k1 = halfDim3;
//              k2 = lenDim3;
//       }
//       else if(region.equals("DSS")) {
//              i1 = halfDim1;
//              i2 = lenDim1;
//
//              j1 = 0;
//              j2 = halfDim2;
//
//              k1 = 0;
//              k2 = halfDim3;
//       }
//       else if(region.equals("DSD")) {
//              i1 = halfDim1;
//              i2 = lenDim1;
//
//              j1 = 0;
//              j2 = halfDim2;
//
//              k1 = halfDim3;
//              k2 = lenDim3;
//       }
//       else if(region.equals("DDS")) {
//              i1 = halfDim1;
//              i2 = lenDim1;
//
//              j1 = halfDim2;
//              j2 = lenDim2;
//
//              k1 = 0;
//              k2 = halfDim3;
//       }
//       else if(region.equals("DDD")) {
//              i1 = halfDim1;
//              i2 = lenDim1;
//
//              j1 = halfDim2;
//              j2 = lenDim2;
//
//              k1 = halfDim3;
//              k2 = lenDim3;
//       }
//       else {
//          System.out.println("Unrecognized region: <" + region +">");
//          System.exit(0);
//       }
//
//
//       get_min_max(data, i1,i2,j1,j2,k1,k2);
//
//       int index=0;
//       for (int i=i1; i<i2; i++) {
//          for (int j=j1; j<j2; j++) {
//             for (int k=k1; k<k2; k++) {
//                 index = (int)((data[i][j][k]- min)/binSpacing);
//                 if (index >= num_bins) {
//                    index = num_bins-1;
//                 }
//                 counts[index]++;
//             }
//          }
//       }
//
//       System.out.println("Created histogram for region <" + region +">");
//    }


    ///////////////////////////////////////////////////////////////////
    // get_min_max
    //
    //
    // flaot ignore - IGNORES that float value anywhere it is encountered
    //                    about
    ///////////////////////////////////////////////////////////////////
    void get_min_max( float data[][][], float IGNORE )
    {
        int dim1 = data.length;
        int dim2 = data[0].length;
        int dim3 = data[0][0].length;

        max = Float.MIN_VALUE;
        min = Float.MAX_VALUE;
 
        for (int i=0; i<dim1; i++) {
            for (int j=0; j<dim2; j++) {
                for (int k=0; k<dim3; k++) {

                   if ( data[i][j][k] != IGNORE ) {
                      if (data[i][j][k] > max)
                         max = data[i][j][k];
                      else if (data[i][j][k] < min)
                         min = data[i][j][k];
                   }

               }
           }
       }      
        binSpacing = (max - min)/num_bins;
        System.out.println(" min = " + min + " max = " + max);
    }


//    ///////////////////////////////////////////////////////////////////
//    // get_min_max
//    //
//    // args
//    //    float data[][][]
//    //    i1, i2 - start and end slice indeces
//    //    j1, j2 - row indeces
//    //    k1, k2 - col indeces
//    //
//    ///////////////////////////////////////////////////////////////////
//    void get_min_max(float data[][][], int i1, int i2, int j1, int j2,
//                     int k1, int k2)
//    {
//        max = Float.MIN_VALUE;
//        min = Float.MAX_VALUE;
//
//        for (int i=i1; i<i2; i++) {
//            for (int j=j1; j<j2; j++) {
//                for (int k=k1; k<k2; k++) {
//                   if (data[i][j][k] != IGNORE) {
//                      if (data[i][j][k] > max)
//                         max = data[i][j][k];
//                      else if (data[i][j][k] < min)
//                        min = data[i][j][k];
//                   }
//                }
//            }
//        }
//
//        binSpacing = (max - min)/num_bins;
//    }

    /**********************************************
     * output_histogram
     *
     * @param filename
     *********************************************/
    void output_histogram(String filename){
        String temp;

        // if no filename - just output to screen
        if (filename == null) {
            for(int i=0; i<num_bins; i++) {
                System.out.println((min + binSpacing*(i+1)) + " , " + counts[i]);
            }
        }
        else {
          try {
             FileWriter fstream = new FileWriter(filename);
             BufferedWriter bw = new BufferedWriter(fstream);

             for(int i=0; i<num_bins; i++) {
                temp = (min + binSpacing*(i+1)) + " , " + counts[i] + "\n";
                bw.write(temp);                 
            }
            bw.flush();
            bw.close();
            fstream.close();
          }
          catch (Exception e) {

          }
        }
    }


//    //------------------------------------------------------
//    //  generateMappedBins(int numMapBins)
//    //
//    //
//    //-------------------------------------------------
//    float[][] generateMappedBins(int numMapBins, mapping mapp) {
//
//       int zero_bin=127;
//
//       // map -
//       //   First value - specifies end location of this bin
//       //   Second value - indicates value to map items in this bin to
//       float[][] map = new float[numMapBins][2];
//
//       for (int i=0; i<numMapBins; i++) {
//          map[i][0] = 0.0f;
//       }
//
//       int numBinsLeft = 0;
//       int numValsLeft = 0;
//       // curBin = current index in map array
//
//
//       // want to lock all bins > avg binSize
//       boolean binLocked[] = new boolean[num_bins];
//
//       int numLocked=0;
//
//       // find 0 bin
//       // - lock 15 before and 15 after
//       int index = (int)(Math.abs((float)min)/(float)binSpacing);
//
//       for (int i=0; i<index-15; i++) {
//          numValsLeft += counts[i];
//       }
//
//       int j=index-16, offset, count;
//
//       numBinsLeft = (numMapBins - 31)/2;
//       int curBin= numBinsLeft - 1;
//
//       int binSize = (int)Math.ceil((float)numValsLeft / (float)numBinsLeft);
//       count=0;
//       offset=0; // offset keeps track of when we've gone over or under
//                     // the target bin size
//
//       int binToMapTo = j;
//       //---------------------------------------------
//       // Fill bins from min up to first locked bin
//       //---------------------------------------------
//       while (numBinsLeft > 1) {
//
//           if ( count < binSize  ){
//              count += counts[j];
//              j--;
//           }
//           else {
//              map[curBin][0] =  min + binSpacing*(binToMapTo+1);
//              //System.out.print(curBin+": "+ map[curBin] + "   j= "+j + "    count= "+ count);
//               // update values
//              curBin--;
//              numBinsLeft--;
//              numValsLeft -= count;
//
//              binSize = (int)Math.ceil((float)numValsLeft / (float)numBinsLeft);
//              offset = binSize - count;
//
//              //System.out.println("   numbinsLeft=" + numBinsLeft
//              //                + "   numValsLeft=" + numValsLeft
//              //                + "   binSize = " + binSize
//              //                + "   offset =" + offset);
//
//              count = 0;
//              binToMapTo=j;
//           }
//        }
//        map[curBin][0] = min + binSpacing*(binToMapTo+1);
//
//        //-------------------------------
//        // Fill locked bins near zero
//        //-------------------------------
//         numBinsLeft = (numMapBins - 31)/2;
//
//         curBin= numBinsLeft;
//         int k = index-15;
//         for (int i=0; i<31; i++) {
//            map[curBin][0] = min + binSpacing*(k+1);
//            curBin++;
//            k++;
//         }
//
//       //---------------------------------------------
//       // Fill bins from last locked bin to last bin
//       //---------------------------------------------
//       // numBinsLeft = (numMapBins - 31)/2; // was set just under prev comment
//       if (numBinsLeft%2 == 0)
//             numBinsLeft++;
//
//       numValsLeft=0;
//       for (int i=index+16; i<num_bins; i++) {
//          numValsLeft += counts[i];
//       }
//
//       binSize = (int)Math.ceil((float)numValsLeft / (float)numBinsLeft);
//       j =index+16;
//       curBin= numMapBins - numBinsLeft;
//
//       //System.out.println("**: j= "+j+ "     curBin=" + curBin   + "     count="
//       //                                     + count + "    binSize =" + binSize
//       //                                     + "   numBinsLeft=" + numBinsLeft
//       //                                     + "   numValsLeft=" + numValsLeft);
//
//       while (numBinsLeft > 1) {
//
//           if ( count < binSize  ){
//              count += counts[j];
//              j++;
//           }
//           else {
//              map[curBin][0] =  min + binSpacing*j;
//              //System.out.print(curBin+": "+ map[curBin] + "   j= "+j + "    count= "+ count);
//               // update values
//              curBin++;
//              numBinsLeft--;
//              numValsLeft -= count;
//
//              binSize = (int)Math.ceil((float)numValsLeft / (float)numBinsLeft);
//              offset = binSize - count;
//
//              //System.out.println("   numbinsLeft=" + numBinsLeft
//              //                + "   numValsLeft=" + numValsLeft
//              //                + "   binSize = " + binSize
//              //                + "   offset =" + offset);
//
//              count = 0;
//           }
//        }
//        map[curBin][0] = max;
//
//
//        // Assign mapped value to each bin
//        if (mapp == mapping.MID) {
//           mapToMid(numMapBins, map);
//        }
//        else {
//
//        }
//
//        map[zero_bin][1]=0.0f;
//
//        return map;
//    }


    //------------------------------------------------------
    //  generateMappedBins(int numMapBins)
    //
    //
    //-------------------------------------------------
    float[][] generateMappedBins(int numMapBins, mapping mapp, int NUM_LOCK_SIDE) {

       int zero_bin=127;
       int curBin=0;

       // map -
       //   First value - specifies end location of this bin
       //   Second value - indicates value to map items in this bin to
       float[][] map = new float[numMapBins][2];

       for (int i=0; i<numMapBins; i++) {
          map[i][0] = 0.0f;
       }

       int numBinsLeft = 0;
       int numValsLeft = 0;
       // curBin = current index in map array

       // find 0 bin
       // - lock bins before and after
       int index = (int)(Math.abs((float)min)/(float)binSpacing);
       
     


        int k = index-NUM_LOCK_SIDE;
        float newBinSpacing = ( binSpacing*k)/(zero_bin - NUM_LOCK_SIDE + 1);
        //System.out.println( " newbinSpacing = " + newBinSpacing );
       
        numBinsLeft = (numMapBins - (NUM_LOCK_SIDE*2+1) )/2;
       //---------------------------------------------
       // Fill bins from min up to first locked bin
       //---------------------------------------------
       curBin = 0;
       while (numBinsLeft > 0) {

              map[curBin][0] =  min + newBinSpacing*(curBin+1);
              System.out.println("setting bin " + curBin + "  to " + map[curBin][0] );
              curBin++;
              numBinsLeft--;
        }

        //-------------------------------
        // Fill locked bins near zero
        //-------------------------------
         numBinsLeft = (numMapBins - (NUM_LOCK_SIDE*2+1))/2;

         curBin= numBinsLeft;
         k = index-NUM_LOCK_SIDE;
         for (int i=0; i<(NUM_LOCK_SIDE*2+1); i++) {
            map[curBin][0] = min + binSpacing*(k+1);
            System.out.println("setting bin " + curBin + "  to " + map[curBin][0] );
            curBin++;
            k++;
         }


       //---------------------------------------------
       // Fill bins from last locked bin to last bin
       //---------------------------------------------
       numBinsLeft = (numMapBins - (NUM_LOCK_SIDE*2+1))/2;
       if ((numMapBins - (NUM_LOCK_SIDE*2+1))%2 != 0) {
      
          numBinsLeft++;
        }
       k = index+(NUM_LOCK_SIDE+1);
       newBinSpacing = ( binSpacing*(num_bins - k))/numBinsLeft;
       //System.out.println( " newbinSpacing = " + newBinSpacing );

       while (numBinsLeft > 0) {
              map[curBin][0] = map[curBin-1][0] + newBinSpacing;
              System.out.println("setting bin " + curBin + "  to " + map[curBin][0] );
              curBin++;
              numBinsLeft--;
       }


        // Assign mapped value to each bin
        if (mapp == mapping.MID) {
           mapToMid(numMapBins, map);
        }
        else {

        }

        map[zero_bin][1]=0.0f;

        return map;
    }


    //------------------------------------------------------
    //  generateMappedBins(int numMapBins)
    //
    //
    //-------------------------------------------------
    void printErrorPerBin(int numMapBins, float [][][] data,
                                  float [][] map, float IGNORE)
    {
       // each bin has four values
        //  - first is Summed Error
        //  - second is Summed Squared Error
        //  - third is Average Error
        //  - fourth is max error
       float[][] error = new float[numMapBins][4];
       int [] count = new int[numMapBins];

       float val=0.0f, temp = 0.0f;
       int index=0;
       float absErr, sqrErr, maxErr;

       int dim1 = data.length;
       int dim2 = data[0].length;
       int dim3 = data[0][0].length;


       SegmentedMapper dm = new SegmentedMapper(map);
       
       for (int i=0; i<dim1; i++) {
           for (int j=0; j< dim2; j++) {
               for (int k=0; k<dim3; k++) {

                   val = data[i][j][k];
                   if (val != IGNORE) {
                       // get index
                       index = dm.getBinFromValue(val);

                       if ( ( index < 0 ) || ( index >= numMapBins )) {
                           System.out.println("  ERROR: index out of bounds!"
                                   + " Can't calculate error per bin value. "
                                   + " <val= "+ val + " >");
                           System.exit(0);
                       }

                       //System.out.println("i=" +i+ "j=" + j+" k="+k + " index = " + index);

                       // compare mapped value to actual and calc errors
                       temp = map[index][1];

                       absErr = Math.abs(val - temp);
                       sqrErr = absErr * absErr;

                       error[index][0] += absErr;
                       error[index][1] += sqrErr;                 
                       count[index]++;
                       error[index][2] = (error[index][0])/((float)count[index]);
                       if (absErr > error[index][3]) {
                           error[index][3] = absErr;
                       }
                       
                   }
               }
           }
       }

       // print error per bin info
       for (int i=0; i< numMapBins;i++) {
          System.out.println("Bin " + i + ", "  + map[i][0] + ", "
                           + map[i][1] + ",   "
                           + error[i][0] + ", "
                           + error[i][1] + ", " + error[i][2] + ", "
                           + error[i][3] );

       }

    }

    //------------------------------------------------------
    //  void mapToMid(int numMapBins, float map[][])
    //
    //
    //-------------------------------------------------
    void mapToMid(int numMapBins, float map[][]){

        float prev=min;

        for (int i=0;i<numMapBins;i++) {
           map[i][1] = (map[i][0] + prev)/2;
           prev = map[i][0];
        }

        return;
    }



//    //------------------------------------------------------
//    //  generateMappedBins(int numMapBins)
//    //
//    //
//    //-------------------------------------------------
//    float[] generateMappedBins(int numMapBins) {
//
//       // map - specifies end location of this bin
//       float[] map = new float[numMapBins];
//
//       for (int i=0; i<numMapBins; i++) {
//          map[i] = 0.0f;
//       }
//
//       int numBinsLeft = numMapBins;
//       int numValsLeft = numVals;
//       // curBin = current index in map array
//       int curBin=0;
//
//       int binSize = (int)Math.ceil((float)numValsLeft / (float)numBinsLeft);
//
//       // want to lock all bins > avg binSize
//       boolean binLocked[] = new boolean[num_bins];
//
//       int numLocked=0;
//       for (int i=0;i<num_bins; i++) {
//          if (counts[i] >= binSize) {
//              binLocked[i] =true;
//              numValsLeft -= counts[i];
//              numBinsLeft--;
//              numLocked++;
//              System.out.println( " locked bin " + i + ":  " + (min + binSpacing*(i+1)) );
//          }
//          else
//              binLocked[i]=false;
//       }
//
//        System.out.println("   numLocked= "+numLocked);
//       if (numBinsLeft < 0) {
//           System.out.println("   Histogram: numBinsLeft<0!  ABORTING");
//           System.exit(0);
//       }
//
//
//       binSize = (int)Math.ceil((float)numValsLeft / (float)numBinsLeft);
//
//       int count=0;
//       int j=0;
//       int offset=0; // offset keeps track of when we've gone over or under
//                     // the target bin size
//       int numSubBins=0;
//       while (numBinsLeft > 1) {
//           System.out.println("**: j= "+j+ "     curBin=" + curBin   + "     count="
//                              + count + "    binSize =" + binSize
//                              + "   numbinsLeft=" + numBinsLeft
//                              + "   numValsLeft=" + numValsLeft);
//           if ( (  count < (binSize + offset)*0.90 ) && (binLocked[j] == false)){
//              count += counts[j];
//              j++;
//              numSubBins++;
//           }
//           else {
//              System.out.println("222222222222222222222222222222222");
//              if ( numSubBins > 0 ) {
//                 // really binsSpacing*( (j-1) + 1)
//                 //  -1 because we aren't including the jth bin
//                 // +1 because coverting zero based index to 1
//                 map[curBin] =  min + binSpacing*(j);
//                 curBin++;
//                 System.out.println(curBin+": "+ map[curBin-1] + "   j= "+j + "    count= "+ count
//                      + "   numbinsLeft=" + numBinsLeft
//                              + "   numValsLeft=" + numValsLeft
//                              + "   binSize = " + binSize
//                              + "   offset =" + offset);
//
//                 // update values
//                 numBinsLeft--;
//                 numValsLeft -= count;
//
//              }
//
//              if ( binLocked[j]==true) {
//                 j++;
//                 map[curBin] = min + binSpacing*(j);
//                 curBin++;
//                 System.out.println(curBin+":: "+ map[curBin-1] + "   j= "+j + "    count= "+ count
//                      + "   numbinsLeft=" + numBinsLeft
//                              + "   numValsLeft=" + numValsLeft
//                              + "   binSize = " + binSize
//                              + "   offset =" + offset);
//                  // update values
//                 numBinsLeft--;
//                 numValsLeft -= counts[j-1];
//
//              }
//              binSize = (int)Math.ceil((float)numValsLeft / (float)numBinsLeft);
//              offset = binSize - count;
//              numSubBins=0;
//              count = 0;
//           }
//
//       }
//       map[numMapBins-1] = max;
//       System.out.println(curBin+": "+ map[curBin] + "   j= "+j + "    count= "+ count);
//       return map;
//    }
    
}
