/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 *
 * ERROR OCCURRED WITH THE FOLLOWING:
 * - looks like it is probably a floating point roundoff error
 * 
 * Excessive bin in mapper <setBinVal method>
 * -- index = 256 numBins = 256
 *
 * Excessive bin in mapper <getMappedValue method>
 * -- index = 256 numBins = 256  binSpacing=2.5064219E-5
 * -- val = 0.0   min=-0.00641644 max =1.4E-45
 *
 */

/**
 *
 * @author Jon
 */
public class LinearMapper {

   private int numBins;
   float min,max;
   float binSpacing;
   float bins[];

   ////////////////////////////////////////////////////////////////////////
   // ctor
   ////////////////////////////////////////////////////////////////////////
   LinearMapper(){
      numBins = 0;
      binSpacing = 0.0f;
      // set them to values that should immediately be replaced by first value
      // in data
      max = Float.MIN_VALUE;
      min = Float.MAX_VALUE;  
   }

   ////////////////////////////////////////////////////////////////////////
   // void mapCoefficients
   //
   // fills each bin with the appropriate lookup value
   //
   // args:
   //   float arr[][][] - 3d array of coefficients
   //   int binCount - number of bins to assign data
   //
   ////////////////////////////////////////////////////////////////////////
   public void mapCoefficients(float arr[][][], int binCount){
       // set class members
       numBins = binCount;
       bins = new float[numBins];

       // get dimensions of the array
       int dim1 = arr.length;
       int dim2 = arr[0].length;
       int dim3 = arr[0][0].length;
            
       // get min and max
       for (int i=0;i<dim1;i++) {
           for (int j=0;j<dim2;j++){
               for (int k=0;k<dim3;k++){                  
                   if (arr[i][j][k] > max)
                       max = arr[i][j][k];

                   if (arr[i][j][k] < min)
                       min = arr[i][j][k];   
               }
           }
       }
     
       float range = max - min;

       // debug print
       System.out.println("@@@ min = " + min + "  max = " + max + "  range = "
                          + range + " @@@");
       //set bin spacing
       binSpacing = range/numBins;

       // fill lookup table
       float val = min + binSpacing/2;
       for (int i=0;i<numBins;i++){
          bins[i] = val;
          val+= binSpacing;
       }

       setBinVal(0.0f, 0.0f);
   }

   ////////////////////////////////////////////////////////////////////////
   // void mapCoefficients
   //
   // fills each bin with the appropriate lookup value
   //
   // args:
   //   float minVal - lowest value in the data
   //   float maxVal - largest value in the data
   //   int binCount - number of bins to assign data
   //
   //
   //
   ////////////////////////////////////////////////////////////////////////
   public void mapCoefficients(float minVal, float maxVal, int binCount)
   {
       // set class members
       numBins = binCount;
       bins = new float[numBins];
       min = minVal;
       max = maxVal;

       float range = max - min;
       // debug print
       System.out.println("@@@ min = " + min + "  max = " + max + "  range = "
                          + range + " @@@");
       // assign bin spacing
       binSpacing = range/numBins;

       // fill lookup table
       float val= min + binSpacing/2;
       for (int i=0;i<numBins;i++){
          bins[i] = val;
          val+= binSpacing;
       }
   }

   ////////////////////////////////////////////////////////////////////////
   // float getMappedValue
   //
   // return the lookup value result
   //
   // args:
   //   float val - original value
   //
   // returns:
   //   result of the lookup
   //
   ////////////////////////////////////////////////////////////////////////
   public float getMappedValue(float val){

       if (val == max) 
           return bins[numBins-1];

       int index = (int)((val-min)/binSpacing);

       if ( index == numBins ) {
           index = numBins - 1;
       }
       if ( index > numBins ) {
           System.err.println( "Excessive bin in mapper"
                             + " <getMappedValue method>" );
           System.err.println( " -- index = " + index
                             + " numBins = " + numBins
                             + "  binSpacing=" + binSpacing);
           System.err.println( " -- val = " + val   + "   min=" + min   + " max ="+max);
           index = numBins - 1;
           System.exit(0);
        }
       return bins[index];
   }

   ////////////////////////////////////////////////////////////////////////
   // void setBinVal
   //
   // sets value of bin containing lookup value
   //
   // args:
   //   float lookupVal - value to find the bin for
   //   float newVal    - new value to assign to bin containing lookupVal
   //
   // returns:
   //   none
   //
   ////////////////////////////////////////////////////////////////////////
   public void setBinVal(float lookupVal, float newVal){

       if (lookupVal == max) {
           bins[numBins-1] = newVal;
           return;
       }

       int index = (int)((lookupVal-min)/binSpacing);
       if ( index >= numBins ) {
           System.err.println( "Excessive bin in mapper <setBinVal method>" );
           System.err.println( " -- index = " + index
                             + " numBins = " + numBins);
           index = numBins - 1;
        }
       bins[index] = newVal;
   }

}
