//****************************************************************************
// Region.java
//
// 
//
// author: Jon Frain
//
//**************************************************************************** 
import java.io.*;
import java.util.*;

public class Region {

   public enum region 
   {
        SSS(0), SSD(1), SDS(2),SDD(3), DSS(4), DSD(5), DDS(6), DDD(7);

        region(int v){
           this.value = v;
        }
        private final int value;
       
        public int getValue(){ return this.value;};

        public static region getRegion(int val) {
           switch (val) {
              case 0:
                 return region.SSS;
              case 1:
                 return region.SSD;
              case 2:
                 return region.SDS;
              case 3:
                 return region.SDD;
              case 4:
                 return region.DSS;
              case 5:
                 return region.DSD;
              case 6:
                 return region.DDS;
              case 7:
                 return region.DDD;
              default:
                  System.out.println( "Unable to return region for val = "+val );
                 return null;
           }
        };

        public static String getRegionString(int val) {
           switch (val) {
              case 0:
                 return "SSS";
              case 1:
                 return "SSD";
              case 2:
                 return "SDS";
              case 3:
                 return "SDD";
              case 4:
                 return "DSS";
              case 5:
                 return "DSD";
              case 6:
                 return "DDS";
              case 7:
                 return "DDD";
              default:
                 return null;
           }
        };
   };

   // CONSTRUCTOR
   private Region( ) {

   }


    //-------------------------------------------------------------------------
    // setDetailRegionZeros(float arr[][][], region dr)
    //
    //
    //-------------------------------------------------------------------------
    public static void setDetailRegionZeros(float arr[][][], region dr,int lvl)
    {
        setDetailRegionVal(arr, dr,0.0f, lvl);
    }



    //-------------------------------------------------------------------------
    // setDetailRegionZeros(float arr[][][], region dr)
    //
    //
    //-------------------------------------------------------------------------
    public static void setDetailRegionZeros(float arr[][][], region dr,
                                            int start1, int start2, int start3, int lvl)
    {
        setDetailRegionVal(arr, dr,start1, start2, start3,0.0f, lvl);
    }


    //-------------------------------------------------------------------------
    // setDetailRegionVal
    //
    // Args:
    //
    //    float arr[][][] - 3D array to modify
    //    region dr       - 3D region to modify (ie SSS, DDD, SSD, ...)
    //    float val       - value to set each number in the region to
    //    int lvl         - Which level to apply it to
    //
    //                 lvl 1 - regions after one level of deconstruction
    //                         in which each region takes up (1/8) of the
    //                         entire 3D space.
    //
    //                         2D Analogy:
    //
    //                            -----------------------
    //                            |          |          |
    //                            |   SS     |    SD    |
    //                            |          |          |
    //                            -----------------------
    //                            |          |          |
     //                           |   DS     |    DD    |
    //                            |          |          |
    //                            -----------------------
    //
    //
    //                 lvl 2 - regions generated after two levels of
    //                         deconstruction.  Each region is now in the
    //                         space occupied by SSS in lvl 1
    //
    //                         2D Analogy:
    //
    //                            ------------------------
    //                            | SS2 | SD2 |          |
    //                            |-----------|          |
    //                            | DS2 | DD2 |          |
    //                            ------------------------
    //                            |           |          |
     //                           |           |          |
    //                            |           |          |
    //                            ------------------------
    //
    //-------------------------------------------------------------------------
    public static void setDetailRegionVal(float arr[][][], region dr,
                                          float val, int lvl)
    {

       if (dr == null) {
          System.out.println("ERROR: setDetailRegionVal() arg dr is null ");
          System.exit(1);
       }

       if (lvl == 0) {
          System.out.println("ERROR: setDetailRegionVal() arg lvl is 0 ");
          System.out.println("-- Aborting program as this would cause 2 raised to a negative value");
          System.exit(1);
       }

       int dividend = (int)( Math.pow( 2,(lvl-1) ) );

       // determine the size in each dimension
       int lenDim1 = (arr.length)/dividend;
       int lenDim2 = (arr[0].length)/dividend;
       int lenDim3 = (arr[0][0].length)/dividend;

       int halfDim1 = (lenDim1/2);
       int halfDim2 = (lenDim2/2);
       int halfDim3 = (lenDim3/2);


//       System.out.println(" (lenDim1, lendim2, lendim3) = (" + lenDim1 + " , "
//                          + lenDim2 + " , " + lenDim3 + ")");
//
//      System.out.println(" (halfDim1, halfdim2, halfdim3) = (" + halfDim1 + " , "
//                          + halfDim2 + " , " + halfDim3 + ")");

       int i1, i2; // slice
       int j1, j2; // row
       int k1, k2; // col

       i1=i2=j1=j2=k1=k2=0;

       switch (dr) {
          case SSS:
              i1 = 0;
              i2 = halfDim1;

              j1 = 0;
              j2 = halfDim2;

              k1 = 0;
              k2 = halfDim3;
              break;
          case SSD:
              i1 = 0;
              i2 = halfDim1;

              j1 = 0;
              j2 = halfDim2;

              k1 = halfDim3;
              k2 = lenDim3;
              break;
          case SDS:
              i1 = 0;
              i2 = halfDim1;

              j1 = halfDim2;
              j2 = lenDim2;

              k1 = 0;
              k2 = halfDim3;
              break;
          case SDD:
              i1 = 0;
              i2 = halfDim1;

              j1 = halfDim2;
              j2 = lenDim2;

              k1 = halfDim3;
              k2 = lenDim3;
              break;
          case DSS:
              i1 = halfDim1;
              i2 = lenDim1;

              j1 = 0;
              j2 = halfDim2;

              k1 = 0;
              k2 = halfDim3;
              break;
          case DSD:
              i1 = halfDim1;
              i2 = lenDim1;

              j1 = 0;
              j2 = halfDim2;

              k1 = halfDim3;
              k2 = lenDim3;
              break;
          case DDS:
              i1 = halfDim1;
              i2 = lenDim1;

              j1 = halfDim2;
              j2 = lenDim2;

              k1 = 0;
              k2 = halfDim3;
              break;
           case DDD:
              i1 = halfDim1;
              i2 = lenDim1;

              j1 = halfDim2;
              j2 = lenDim2;

              k1 = halfDim3;
              k2 = lenDim3;
              break;
       }

       for (int i=i1; i<i2; i++) {
          for (int j=j1; j<j2; j++) {
             for (int k=k1; k<k2; k++) {
                 arr[i][j][k] = val;
             }
          }
       }

    }

    //-------------------------------------------------------------------------
    // copyDetailRegionValues
    //
    //  Assumptions: Source and destination arrays are the same dimensions!
    //
    // Args:
    //
    //    float source[][][] - 3D array to copy from
    //    float dest[][][] - 3D array to copy to
    //    region dr       - 3D region to modify (ie SSS, DDD, SSD, ...)
    //    int lvl         - Which level to apply it to
    //
    //                 lvl 1 - regions after one level of deconstruction
    //                         in which each region takes up (1/8) of the
    //                         entire 3D space.
    //
    //                         2D Analogy:
    //
    //                            -----------------------
    //                            |          |          |
    //                            |   SS     |    SD    |
    //                            |          |          |
    //                            -----------------------
    //                            |          |          |
     //                           |   DS     |    DD    |
    //                            |          |          |
    //                            -----------------------
    //
    //
    //                 lvl 2 - regions generated after two levels of
    //                         deconstruction.  Each region is now in the
    //                         space occupied by SSS in lvl 1
    //
    //                         2D Analogy:
    //
    //                            ------------------------
    //                            | SS2 | SD2 |          |
    //                            |-----------|          |
    //                            | DS2 | DD2 |          |
    //                            ------------------------
    //                            |           |          |
     //                           |           |          |
    //                            |           |          |
    //                            ------------------------
    //
    //-------------------------------------------------------------------------
    public static void copyRegionValues(float dest[][][],
                                        float source[][][],
                                        region dr,
                                        int lvl)
    {

       if (dr == null) {
          System.out.println("ERROR: copyDetailRegionValues() arg dr is null ");
          System.exit(1);
       }

       int dividend = (int)( Math.pow( 2, (lvl-1) ) );

       // determine the size in each dimension
       int lenDim1 = (source.length)/dividend;
       int lenDim2 = (source[0].length)/dividend;
       int lenDim3 = (source[0][0].length)/dividend;

       int halfDim1 = (lenDim1/2);
       int halfDim2 = (lenDim2/2);
       int halfDim3 = (lenDim3/2);

       int i1, i2; // slice
       int j1, j2; // row
       int k1, k2; // col

       i1=i2=j1=j2=k1=k2=0;

       switch (dr) {
          case SSS:
              i1 = 0;
              i2 = halfDim1;

              j1 = 0;
              j2 = halfDim2;

              k1 = 0;
              k2 = halfDim3;
              break;
          case SSD:
              i1 = 0;
              i2 = halfDim1;

              j1 = 0;
              j2 = halfDim2;

              k1 = halfDim3;
              k2 = lenDim3;
              break;
          case SDS:
              i1 = 0;
              i2 = halfDim1;

              j1 = halfDim2;
              j2 = lenDim2;

              k1 = 0;
              k2 = halfDim3;
              break;
          case SDD:
              i1 = 0;
              i2 = halfDim1;

              j1 = halfDim2;
              j2 = lenDim2;

              k1 = halfDim3;
              k2 = lenDim3;
              break;
          case DSS:
              i1 = halfDim1;
              i2 = lenDim1;

              j1 = 0;
              j2 = halfDim2;

              k1 = 0;
              k2 = halfDim3;
              break;
          case DSD:
              i1 = halfDim1;
              i2 = lenDim1;

              j1 = 0;
              j2 = halfDim2;

              k1 = halfDim3;
              k2 = lenDim3;
              break;
          case DDS:
              i1 = halfDim1;
              i2 = lenDim1;

              j1 = halfDim2;
              j2 = lenDim2;

              k1 = 0;
              k2 = halfDim3;
              break;
           case DDD:
              i1 = halfDim1;
              i2 = lenDim1;

              j1 = halfDim2;
              j2 = lenDim2;

              k1 = halfDim3;
              k2 = lenDim3;
              break;
       }


//       System.out.println( "============= Copying Values =================");
//       System.out.println( "     i1="+i1+"   i2="+i2
//                         + "     j1="+j1+"   j2="+j2
//                         + "     k1="+k1+"   k2="+k2);
//       System.out.println( "==============================================");
       
       for (int i=i1; i<i2; i++) {
          for (int j=j1; j<j2; j++) {
             for (int k=k1; k<k2; k++) {
                 dest[i][j][k] = source[i][j][k];
             }
          }
       }

    }

    
    //-------------------------------------------------------------------------
    // setDetailRegionVal
    //
    // Args:
    //
    //    float arr[][][] - 3D array to modify
    //    region dr        - 3D region to modify (ie SSS, DDD, SSD, ...)
    //    float val       - value to set each number in the region to
    //    int start1, start2, start3
    //                     - detail regions relative to starting lcoation
    //
    //    int lvl         - Which level to apply it to
    //
    //                 lvl 1 - regions after one level of deconstruction
    //                         in which each region takes up (1/8) of the
    //                         entire 3D space.
    //
    //                         2D Analogy:
    //
    //                            -----------------------
    //                            |          |          |
    //                            |   SS     |    SD    |
    //                            |          |          |
    //                            -----------------------
    //                            |          |          |
     //                           |   DS     |    DD    |
    //                            |          |          |
    //                            -----------------------
    //
    //
    //                 lvl 2 - regions generated after two levels of
    //                         deconstruction.  Each region is now in the
    //                         space occupied by SSS in lvl 1
    //
    //                         2D Analogy:
    //
    //                            -------------------------
    //                            | SS2 | SD2 | SS2 | SD2 |
    //                            |-----------|-----------|
    //                            | DS2 | DD2 | DS2 | DD2 |
    //                            -------------------------
    //                            | SS2 | SD2 | SS2 | SD2 |
    //                            |-----------|-----------|
    //                            | DS2 | DD2 | DS2 | DD2 |
    //                            -------------------------
    //
    //-------------------------------------------------------------------------
    public static void setDetailRegionVal(float arr[][][], region dr,
                                          int start1, int start2, int start3,
                                          float val, int lvl)
    {

       if (dr == null) {
          System.out.println("ERROR: setDetailRegionVal() arg dr is null ");
          System.exit(1);
       }

       if (lvl == 0) {
          System.out.println("ERROR: setDetailRegionVal() arg lvl is 0 ");
          System.out.println("-- Aborting program as this would cause div by 0");
          System.exit(1);
       }

       int dividend = (int)( Math.pow( 2,(lvl-1) ) );

       // determine the size in each dimension
       int lenDim1 = (arr.length)/dividend;
       int lenDim2 = (arr[0].length)/dividend;
       int lenDim3 = (arr[0][0].length)/dividend;

       int halfDim1 = (lenDim1/2);
       int halfDim2 = (lenDim2/2);
       int halfDim3 = (lenDim3/2);

       int i1, i2; // slice
       int j1, j2; // row
       int k1, k2; // col

       i1=i2=j1=j2=k1=k2=0;

       switch (dr) {
          case SSS:
              i1 = start1+0;
              i2 = start1+halfDim1;

              j1 = start2+0;
              j2 = start2+halfDim2;

              k1 = start3+0;
              k2 = start3+halfDim3;
              break;
          case SSD:
              i1 = start1+0;
              i2 = start1+halfDim1;

              j1 = start2+0;
              j2 = start2+halfDim2;

              k1 = start3+halfDim3;
              k2 = start3+lenDim3;
              break;
          case SDS:
              i1 = start1+0;
              i2 = start1+halfDim1;

              j1 = start2+halfDim2;
              j2 = start2+lenDim2;

              k1 = start3+0;
              k2 = start3+halfDim3;
              break;
          case SDD:
              i1 = start1+0;
              i2 = start1+halfDim1;

              j1 = start2+halfDim2;
              j2 = start2+lenDim2;

              k1 = start3+halfDim3;
              k2 = start3+lenDim3;
              break;
          case DSS:
              i1 = start1+halfDim1;
              i2 = start1+lenDim1;

              j1 = start2+0;
              j2 = start2+halfDim2;

              k1 = start3+0;
              k2 = start3+halfDim3;
              break;
          case DSD:
              i1 = start1+halfDim1;
              i2 = start1+lenDim1;

              j1 = start2+0;
              j2 = start2+halfDim2;

              k1 = start3+halfDim3;
              k2 = start3+lenDim3;
              break;
          case DDS:
              i1 = start1+halfDim1;
              i2 = start1+lenDim1;

              j1 = start2+halfDim2;
              j2 = start2+lenDim2;

              k1 = start3+0;
              k2 = start3+halfDim3;
              break;
           case DDD:
              i1 = start1+halfDim1;
              i2 = start1+lenDim1;

              j1 = start2+halfDim2;
              j2 = start2+lenDim2;

              k1 = start3+halfDim3;
              k2 = start3+lenDim3;
              break;
       }
//       System.out.println("===================================================");
//       System.out.println("  Setting detail region--" +dr+"-- to <<"+val+">>");
//       System.out.println("      <<< lvl = " + lvl + ">>>");
//       System.out.println("      ----  <slice>i1 = " + i1 + " i2 = " + i2);
//       System.out.println("      ----  <row>  j1 = " + j1 + " j2 = " + j2);
//       System.out.println("      ----  <col>  k1 = " + k1 + " k2 = " + k2);
//      System.out.println("===================================================\n\n");


       for (int i=i1; i<i2; i++) {
          for (int j=j1; j<j2; j++) {
             for (int k=k1; k<k2; k++) {
                 arr[i][j][k] = val;
             }
          }
       }

    }


   //-------------------------------------------------------------------------
    // getRegionFromLoc(int xLoc, int yLoc, int zLoc,
    //                  int dim1, int dim2, int dim3)
    //
    //
    //-------------------------------------------------------------------------
    public static int getRegionFromLoc(int xLoc, int yLoc, int zLoc,
                                 int dim1, int dim2, int dim3)
    {
       int halfDim1 = (dim1/2);
       int halfDim2 = (dim2/2);
       int halfDim3 = (dim3/2);

       if (xLoc < halfDim1) {
           if (yLoc < halfDim2) {
               if (zLoc < halfDim3) {
                   // region.SSS
                   return 0;
               }
               else {
                   // region.SSD
                   return 1;
               }
           }
           else {
              if (zLoc < halfDim3) {
                   // region.SDS
                   return 2;
               }
               else {
                   // region.SDD
                   return 3;
               }
           }
       }
       else {
           if (yLoc < halfDim2) {
               if (zLoc < halfDim3) {
                   // region.DSS
                   return 4;
               }
               else {
                   // region.DSD
                   return 5;
               }
           }
           else {
              if (zLoc < halfDim3) {
                   // region.DDS
                   return 6;
               }
               else {
                   // region.DDD
                   return 7;
               }
           }
       }
    }


   public static String generateLabel(String label){
      String temp="";

      for (int i=0;i<8;i++)
         if ( label.contains(""+i) )
            temp+=  Region.region.getRegionString(i) + "+";

     // remove last dash
     temp = temp.substring(0, (temp.length() - 1));

      return temp;
   }
    

   //--------------------------------------------------------------------
   // main
   //
   //-------------------------------------------------------------------- 
    public static void main(String [ ] args) {


    }
}



