/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

/**
 * Created: 2/24/2011
 * 
 *
 * @author Jon
 */
import java.io.*;

public class SegmentedMapper {

   private int numBins;
   float min,max;
   float binSpacing;
   float bins[][];

   ////////////////////////////////////////////////////////////////////////
   // ctor
   ////////////////////////////////////////////////////////////////////////
   SegmentedMapper(){
      numBins = 256;
      binSpacing = 0.0f;

      // set them to values that should immediately be replaced by first value
      // in data
      max = Float.MIN_VALUE;
      min = Float.MAX_VALUE;  
   }


   ////////////////////////////////////////////////////////////////////////
   // ctor
   //
   // This version is given the 2D map array
   //
   ////////////////////////////////////////////////////////////////////////
   SegmentedMapper(float[][] map){
      numBins=256;
      bins = map;
      min = bins[0][0];
      max = bins[255][0];
      System.out.println(" SM: min = " + min + "  max = " + max);
     
   }


   ////////////////////////////////////////////////////////////////////////
   // void mapCoefficients
   //
   // fills each bin with the appropriate lookup value
   //
   // args:
   //   float arr[][][] - 3d array of coefficients
   //   int binCount - number of bins to assign data
   //
   ////////////////////////////////////////////////////////////////////////
   public void mapCoefficients(float arr[][][], String filename)
   {
       // set class members
       // we have 256 bins - each bin has the endpoint of that bin and its
       // value
       bins = new float[numBins][2];

       System.out.println("Entered map coefficients");
       // get dimensions of the array
       int dim1 = arr.length;
       int dim2 = arr[0].length;
       int dim3 = arr[0][0].length;
            
       // get min and max
       for (int i=0;i<dim1;i++) {
           for (int j=0;j<dim2;j++){
               for (int k=0;k<dim3;k++){                  
                   if (arr[i][j][k] > max)
                       max = arr[i][j][k];

                   if (arr[i][j][k] < min)
                       min = arr[i][j][k];   
               }
           }
       }
     
       // make range symmetric about 0
       if ( Math.abs(min) > max)
           max = -min;
       else
           min = -max;

       float range = max - min;
       // debug print
       System.out.println("@@@ min = " + min + "  max = " + max + "  range = "
                          + range + " @@@");
       // fill lookup table
       // First open file
       FileInputStream in=null;
       DataInputStream din=null;

       try {
          in = new FileInputStream(filename);
          din = new DataInputStream(in);
       }catch (Exception e) {
          System.out.println("Exception thrown when opening or reading from "
                             + " file " + filename );
          e.printStackTrace();
          System.exit(0);
       }

       // for testing - linear
//       float binSpacing = range/numBins;
//
//       // fill lookup table
//       float val = min + binSpacing/2;
//       float end = min + binSpacing;
//
//       for (int i=0; i<numBins; i++){
//          bins[i][0] = end;
//          bins[i][1] = val;
//          val += binSpacing;
//          end += binSpacing;
//       }

       // Read in each endpoint as a % of range
       float temp=0.0f;
       for (int i=0; i<256;i++) {

          try {
             // read next endpoint
             temp = din.readFloat();
             bins[i][0] = min + temp*range;
             if (i == 0) {
                bins[i][1] = (min + bins[i][0])/2;
             }
             else {
                bins[i][1] = (bins[i-1][0] + bins[i][0])/2;
             }
          }
          catch (Exception e) {
             System.out.println("Unable to read float value " + (i+1) +
                                " from mapping file");
             e.printStackTrace();
             System.exit(0);
          }
       }

       // set bin containing 0 equal to 0
       //setBinVal(0.0f, 0.0f);

       int bin = getBinFromValue(0.0f);

       bins[bin][1]=0.0f;
       //bins[bin+1][1]=0.0f;

       System.out.println("Done map coefficients");
        // for (int i=0;i<256;i++) {
        //    System.out.println(bins[i][0] + "      " + bins[i][1]);
        // }
       
   }


   ////////////////////////////////////////////////////////////////////////
   // void mapCoefficients
   //
   // - alternate implementation - simply accepts 1D array with endpoints
   //
   // fills each bin with the appropriate lookup value
   //
   // args:
   //   float arr[][][] - 3d array of coefficients
   //   int binCount - number of bins to assign data
   //
   ////////////////////////////////////////////////////////////////////////
   public void mapCoefficients(float arr[][][], float map[])
   {
//       // set class members
//       // we have 256 bins - each bin has the endpoint of that bin and its
//       // value
//       bins = new float[numBins][2];
//
//       System.out.println("Entered map coefficients");
//       // get dimensions of the array
//       int dim1 = arr.length;
//       int dim2 = arr[0].length;
//       int dim3 = arr[0][0].length;
//
//       // get min and max
//       for (int i=0;i<dim1;i++) {
//           for (int j=0;j<dim2;j++){
//               for (int k=0;k<dim3;k++){
//                   if (arr[i][j][k] > max)
//                       max = arr[i][j][k];
//
//                   if (arr[i][j][k] < min)
//                       min = arr[i][j][k];
//               }
//           }
//       }
//
//       // make range symmetric about 0
//       if ( Math.abs(min) > max)
//           max = -min;
//       else
//           min = -max;
//
//       float range = max - min;
//       // debug print
//       System.out.println("@@@ min = " + min + "  max = " + max + "  range = "
//                          + range + " @@@");
//       // fill lookup table
//       // First open file
//       FileInputStream in=null;
//       DataInputStream din=null;
//
//       try {
//          in = new FileInputStream(filename);
//          din = new DataInputStream(in);
//       }catch (Exception e) {
//          System.out.println("Exception thrown when opening or reading from "
//                             + " file " + filename );
//          e.printStackTrace();
//          System.exit(0);
//       }
//
//       // for testing - linear
////       float binSpacing = range/numBins;
////
////       // fill lookup table
////       float val = min + binSpacing/2;
////       float end = min + binSpacing;
////
////       for (int i=0; i<numBins; i++){
////          bins[i][0] = end;
////          bins[i][1] = val;
////          val += binSpacing;
////          end += binSpacing;
////       }
//
//       // Read in each endpoint as a % of range
//       float temp=0.0f;
//       for (int i=0; i<256;i++) {
//
//          try {
//             // read next endpoint
//             temp = din.readFloat();
//             bins[i][0] = min + temp*range;
//             if (i == 0) {
//                bins[i][1] = (min + bins[i][0])/2;
//             }
//             else {
//                bins[i][1] = (bins[i-1][0] + bins[i][0])/2;
//             }
//          }
//          catch (Exception e) {
//             System.out.println("Unable to read float value " + (i+1) +
//                                " from mapping file");
//             e.printStackTrace();
//             System.exit(0);
//          }
//       }
//
//       // set bin containing 0 equal to 0
//       //setBinVal(0.0f, 0.0f);
//
//       int bin = getBinFromValue(0.0f);
//
//       bins[bin][1]=0.0f;
//       //bins[bin+1][1]=0.0f;
//
//       System.out.println("Done map coefficients");
//        // for (int i=0;i<256;i++) {
//        //    System.out.println(bins[i][0] + "      " + bins[i][1]);
//        // }

   }


   ////////////////////////////////////////////////////////////////////////
   // float getMappedValue
   //
   // return the lookup value result
   //
   // args:
   //   float val - original value
   //
   // returns:
   //   result of the lookup
   //
   ////////////////////////////////////////////////////////////////////////
   public float getMappedValue(float val){

       int index = getBinFromValue(val);
       
       return bins[index][1];
   }

   ////////////////////////////////////////////////////////////////////////
   // float getBinFromValue
   //
   // return the bin that contains this vlaue
   //
   // args:
   //   float val - value to lookup
   //
   // returns:
   //   bin contianing this value
   //
   ////////////////////////////////////////////////////////////////////////
   public int getBinFromValue(float value){

       if (value <= min)
           return 0;
       else if (value >=max)
           return 255;

       int index = 0;

       // BINARY SEARCH
       boolean found=false;
       int low=0, high=255;
       while (found == false) {
           index = (high+low)/2;
                 
           if (value <= bins[index][0]) {
              if (index == 0)
                  found = true;
               else if(value > bins[index - 1][0]) {
                  found = true;
              }
              else {
                high = index;
              }       
           }
           else if (value > bins[index][0]) {
              if (index == 255)
                  found = true;
              else if(value <= bins[index + 1][0]) {
                 found = true;
                 index++;
              }
              else
                 low=index;
           }
       }
       return index;
   }



   ////////////////////////////////////////////////////////////////////////
   // void setBinVal
   //
   // sets value of bin containing lookup value
   //
   // args:
   //   float lookupVal - value to find the bin for
   //   float newVal    - new value to assign to bin containing lookupVal
   //
   // returns:
   //   none
   //
   ////////////////////////////////////////////////////////////////////////
   public void setBinVal(float lookupVal, float newVal){
       int index = getBinFromValue(lookupVal);
       bins[index][1] = newVal;
   }

}
