//****************************************************************************
// TestBench2.java
//
// Running from school
//     "/media/TOSHIBA EXT/MS_PROJECT/data/jimmy_timesteps_384/" endian=LITTLE
//     "/media/TOSHIBA EXT/MS_PROJECT/data/head-float/" endian=BIG
//
// running from windows
//   E:\MS_PROJECT\data\jimmy_timesteps_384\  endian=LITTLE
//   E:\MS_PROJECT\data\jimmy_384\  endian=LITTLE
//   E:\MS_PROJECT\data\head-float\   endian=BIG
//   E:\MS_PROJECT\data\jimmy_000480-576x384x576\ endian=LITTLE
//
//   E:\MS_PROJECT\data\head-float_partitions_4x4x4\ endian=BIG
//
//   E:\MS_PROJECT\data\jimmy_384_partitions_4x4x4\ endian=LITTLE
//
//   E:\MS_PROJECT\data\jimmy_000480-576x192x288\ endian=LITTLE
//   E:\MS_PROJECT\data\jimmy_000480-630x200x300\ endian=LITTLE
//   E:\MS_PROJECT\data\jimmy_000480-576x192x288_partitions_4x4x4 endian=LITTLE
//   E:\MS_PROJECT\data\jimmy_000480_576x192x288_partitions_8x8x8 endian= LITTLE
//
//   E:\MS_PROJECT\data\raf-512x256x256_64partitions\ endian=LITTLE
//   E:\MS_PROJECT\data\raf-512x256x256_512partitions\ endian=LITTLE
//   E:\MS_PROJECT\data\raf-512x256x256\ endian=LITTLE
//
//   E:\MS_PROJECT\data\partitions-test\lvl2\ endian=LITTLE
//
//**************************************************************************** 
import java.io.*;
import java.util.*;

public class TestBench2 {

    // # of levels of transformation
    private static int NUM_LVLS = 1;
    private static int NUM_COMPRESSION_BINS=256;

    // boolean to set whether summary coefficients are to be compressed
    private static boolean COMPRESS_SUMMARY=false;

    // boolean to set whether detail coefficients are to be compressed
    private static boolean COMPRESS_DETAILS=false;
    private String mapFilename="";
    
    // error caused by each region at each level
    protected static int dim;         // dimensionality of data
    protected static String directory;
    protected static float min, max;
    protected static float range;
    private static float[][] unh_region_error;
    private static float[] region_percentages;
    private static float[] classic_region_error_contributions;
    private static float[] classic_region_coefficient_count;

    // sums the coefficients saved from each region
    private static float[] classic_region_coefficient_sum;
    private String logFile, logFile2;
    private static String fileName;
    static private BufferedWriter log, log2;
    private final static int NUM_BINS = 25;

    private static boolean bigEndian;   
    private static int[] bins;
 

    //------------------------------------------
    // CONSTRUCTOR
    //------------------------------------------
    public TestBench2(int d) {
        dim = d;
        directory = "";
        
        // statistics variables
        region_percentages = new float[8];
        classic_region_coefficient_sum = new float[8];
        unh_region_error = new float[NUM_LVLS][8];

        for (int i = 0; i < 8; i++) {
            region_percentages[i] = 0;
            classic_region_coefficient_sum[i] = 0;
            for (int j=0; j<NUM_LVLS; j++){
                unh_region_error[j][i]=0.0f;
            }
        }

        // Open log file
        logFile = "results.csv";
        logFile2 = "results_best.csv";

        try {
            FileWriter fstream = new FileWriter(logFile);
            log = new BufferedWriter(fstream);

            FileWriter fstream2 = new FileWriter(logFile2);
            log2 = new BufferedWriter(fstream2);
        } catch (Exception e) {
            e.printStackTrace();
            System.exit(0);
        }
    }

    
    //==================================================================
    // processCommandArgs(String[] args)
    //
    //
    // Purpose: Get directory containing data and endianness of data
    //
    //==================================================================
    public void processCommandArgs(String[] args) {
        boolean setEndian=false, setMapFilename=false;

        String endian = "";

        bins = new int[NUM_BINS];

        System.out.println(" args.length = " + args.length);

        directory = args[0];

        for (int i=1; i<args.length; i++) {

            // setting endianness ?
            if (args[i].startsWith("endian=") ){

                if (args[i].endsWith("BIG")){
                    bigEndian = true;
                }
                else if(args[i].endsWith("LITTLE"))
                {
                    bigEndian = false;
                }
                else {
                   System.out.println("endian= must be follwed by \"BIG\" or "
                                   + "\"LITTLE\".  Aborting!");
                   System.exit(0);
                }
                setEndian=true;
            }
            // setting number of levels of transforms ?
            else if (args[i].startsWith("NUM_LVLS=") ){
                String num_lvls = args[i].substring(9, args[i].length());
                System.out.println("num_lvls = " + num_lvls);

                try {
                    int val = Integer.parseInt(num_lvls, 10);
                    NUM_LVLS = val;
                    System.out.println("NUM_LVLS = " + NUM_LVLS);
                } catch (NumberFormatException nfe){
                    System.out.println(" Argument Error:  Expecting Integer"
                                       + " after NUM_LVLS=");
                    System.out.println(" --  Using default value of "
                                         + NUM_LVLS
                                         + " levels of transformation");
                  }
            }
            else if (args[i].startsWith("mapFilename="))  {
                mapFilename = args[i].substring(12, args[i].length());
                System.out.println("mapFilename = " + mapFilename);
                setMapFilename=true;
            }
            else if (args[i].startsWith("COMPRESS_SUMMARY="))  {
               if (args[i].endsWith("TRUE")){
                    COMPRESS_SUMMARY = true;
               }
                else if(args[i].endsWith("FALSE"))
                {
                    COMPRESS_SUMMARY = false;
               }
               else {
                  System.out.println("COMPRESS_SUMMARY must be set to TRUE or FALSE");
                  System.out.println("<"+args[i]+">");
                  System.exit(0);
               }
            }
            else if (args[i].startsWith("COMPRESS_DETAILS="))  {
               if (args[i].endsWith("TRUE")){
                    COMPRESS_DETAILS = true;
               }   
               else if(args[i].endsWith("FALSE"))
               {
                    COMPRESS_DETAILS = false;
               }
               else {
                  System.out.println("COMPRESS_DETAILS must be set to TRUE or FALSE");
                  System.out.println("<"+args[i]+">");
                  System.exit(0);
               }
           }

        }

        // make sure we got all of the arguments we need
        if ( setEndian==false ){
            System.out.println("Need endianess set!  Aborting!");
            System.exit(0);
        }
    }

    //-------------------------------------------------------------------
    // ArrayList<String> getListOfFiles
    //
    //--------------------------------------------------------------------
    public ArrayList<String> getListOfFiles() {

        // following snippet taken from
        //    http://www.java-tips.org/java-se-tips/java.io/
        //    list-the-names-of-all-files-in-a-particular-dire.html
        File folder = new File(directory);
        if (!folder.isDirectory()) {
            System.out.println(" Specified folder name <" + directory
                               + "> is not a directory. " +
                               "Aborting!\n");
            System.exit(0);
        }

        File[] listOfFiles = folder.listFiles();
        ArrayList<String> testFilesList = new ArrayList<String>();

        for (int i = 0; i < listOfFiles.length; i++) 
        {
            if (listOfFiles[i].isFile() 
                && listOfFiles[i].getName().endsWith(".bin")) 
            {
               testFilesList.add(listOfFiles[i].getName());
            }
        }

        Collections.sort(testFilesList);
        return testFilesList;
    }

    

 

    //-------------------------------------------------------------------
    // void testFiles
    //
    //--------------------------------------------------------------------
    public void testFiles(ArrayList<String> testFilesList) {

        FileUtil fl = new FileUtil();

        // Create sum squared error array - one for each region
        int sse_sz = (int)Math.pow(8,NUM_LVLS);
        float ssError[] = new float[sse_sz];
        float ssErrorReduct[] = new float[sse_sz]; 

        // initialize region errors to zero
        for (int i=0; i<sse_sz;i++) {
           ssError[i] = 0.0f;
           ssErrorReduct[i] = 0.0f;
        }

        int dim1 = 0, dim2 = 0, dim3 = 0;

        try {
            ListIterator<String> iter = testFilesList.listIterator();

            // Test every file in the directory
            while (iter.hasNext()) {

                fileName = directory + iter.next();
                System.out.println("========================================="
                                   + "=======================");
                System.out.println("  Running Tests on " + fileName);
                System.out.println("========================================="
                                   + "=======================\n");

                // Read data from file
                float initArr3[][][] = fl.readFloatDataFromFile(fileName,
                                                                bigEndian);

                // get the min, max and range - for debugging only
                max   = fl.getMax();
                min   = fl.getMin();
                range = fl.getRange();

                // debug print of range
                System.out.println("range = " + range + "\n");

                // set dimensions of initial data
                dim1 = initArr3.length;
                dim2 = initArr3[0].length;
                dim3 = initArr3[0][0].length;              

                if (testForOddDimTransform(dim1, dim2, dim3) != 0){
                    System.out.println("Attempting to perform a Transform on"
                                       + "data with an odd dimension!"
                                       + " Aborting program.");
                    System.out.println("-- dim1 = " + dim1 + " dim2 = "
                                       + dim2 + " dim3 = "
                                       + dim3 + " number of transform levels = "
                                       + NUM_LVLS);
                    System.exit(0);
                }
                else {
                    System.out.println(" Check was OK!");
                }

                float transformedArr[][][] = new float[dim1][dim2][dim3];
                float compressedArr[][][]  = new float[dim1][dim2][dim3];

                // initialize region errors to zero
                for (int i=0; i<8;i++)
                   ssError[i] = 0.0f;

                // Test all combinations to find the
                ErrorUtility.calcAndPrintRegionErrorsFloat_ALL_COMBINATIONS(
                                          initArr3, ssError,
                                          ssErrorReduct,
                                          transformedArr,
                                          compressedArr, dim1, dim2, dim3,
                                          fileName, range, log, log2,
                                          unh_region_error,
                                          NUM_LVLS,
                                          COMPRESS_SUMMARY,
                                          COMPRESS_DETAILS,
                                          NUM_COMPRESSION_BINS,
                                          mapFilename);


                       
                try {
                    log.write("\n");
                    log2.write("\n");
                } catch (Exception e) {
                    e.printStackTrace();
                    System.exit(0);
                }
            } // while
            log.flush();
            log.close();

            log2.flush();
            log2.close();

            System.out.println("**********  TOTAL DATA SIZE = "
                               + dim1 * dim2 * dim3 + " floats ***********");
            System.out.println("**********  dim1 = " + dim1 + "  dim2 = "
                               + dim2 + "  dim3 = " + dim3 + "***********");
        } catch (Exception e) {
            e.printStackTrace();
            System.exit(0);
        }
    }







    //--------------------------------------------------------------------
    // basicFunctionalityTest1()
    //
    // Purpose:
    //    Creates a 4x4x4 array performs 2 transforms and then 2 inverse
    //    transforms and prints results of each stage.
    //
    //--------------------------------------------------------------------
    public static void basicFunctionalityTest1(){
       int dim1, dim2, dim3;

       dim1 = dim2 = dim3 = 16;

       double[][][] initArr = new double [dim1][dim2][dim3];

       double val=1.0f;
       for (int i=0; i<dim1;i++){
          for (int j=0; j<dim2;j++){
             for (int k=0; k<dim3;k++){
                initArr[i][j][k] = val++;
             }
          }
       }
       
       System.out.println(" Level = 0");
       System.out.println("==========================");

       ArrayUtil.printArrayFormatted(initArr);

       double[][][] arr2 = new double [dim1][dim2][dim3];
       double[][][] arr3 = new double [dim1][dim2][dim3];
       double[][][] arr4 = new double [dim1][dim2][dim3];
       double[][][] arr5 = new double [dim1][dim2][dim3];

       System.out.println(" Level = 1");
       System.out.println("==========================");

       HaarTransform.transform(initArr, arr2, dim1, dim2, dim3 );
       ArrayUtil.printArrayFormatted(arr2);

       System.out.println(" Level = 2");
       System.out.println("==========================");

       HaarTransform.transform(arr2, arr3, dim1/2, dim2/2, dim3/2 );
       ArrayUtil.printArrayFormatted(arr3);

       System.out.println(" Level = 3");
       System.out.println("==========================");

       HaarTransform.transform(arr3, arr4, dim1/4, dim2/4, dim3/4 );
       ArrayUtil.printArrayFormatted(arr4);

       System.out.println(" Level = 4");
       System.out.println("==========================");

       HaarTransform.transform(arr4, arr5, dim1/8, dim2/8, dim3/8 );
       ArrayUtil.printArrayFormatted(arr5);


//       //============
//        // Test setting detail regions to zero
//        //============
//        System.out.println(" Testing region zeroing");
//        System.out.println("==========================");

//        // set this detail region to zeroes for each lvl specified
//        for (int i = 1; i < 8; i++) {
//           ArrayUtil.copyArray(arr3, initArr);
//           for (int j=1; j<3;j++) {
//              ArrayUtil.copyArray(arr3, initArr);
//              System.out.println("######### REGION = " +
//                                 Region.region.getRegion(i) + " #########");
//              Region.setDetailRegionZeros(arr3, Region.region.getRegion(i), j);
//              ArrayUtil.printArray(arr3);
//           }
//        }
    }

    //--------------------------------------------------------------------
    // basicFunctionalityTest2()
    //
    // Purpose:
    //    Creates a 4x4x4 array performs that tests multiple levels of
    //    ignored regions
    //
    //--------------------------------------------------------------------
    public static void basicFunctionalityTest2(TestBench2 tb){

       int dim1, dim2, dim3;

       dim1 = dim2 = dim3 = 16;
       double[][][] initArr = new double [dim1][dim2][dim3];

       double val=1.0f;
       for (int i=0; i<dim1;i++){
          for (int j=0; j<dim2;j++){
             for (int k=0; k<dim3;k++){
                initArr[i][j][k] = val++;
             }
          }
       }

       System.out.println(" Testing for level = 1");
       System.out.println("==========================");

       ArrayUtil.printArrayFormatted(initArr);

       // Create sum squared error array - one for each region
       double ssError[] = new double[8];

       double transformedArr[][][] = new double[dim1][dim2][dim3];
       double compressedArr[][][]  = new double[dim1][dim2][dim3];

       // Test errors for each region at multiple levels
       for ( int lvl = 1; lvl<=NUM_LVLS; lvl++) {
          // initialize region errors to zero
          for (int i=0; i<8;i++) {
             ssError[i] = 0.0f;
          }
          // Determine errors for UNH method
          // Frain-double-float
          //ErrorUtility.calcAndPrintRegionErrors2(initArr, ssError,
          //                            transformedArr,
          //                            compressedArr, dim1, dim2, dim3, lvl,
          //                            fileName,range, log,unh_region_error);
       }

       try {
          log.write("\n");
          log.flush();
          log.close();
       } catch (Exception e) {
          e.printStackTrace();
          System.exit(0);
       } 
    }

    //--------------------------------------------------------------------
    // basicFunctionalityTest1()
    //
    // Purpose:
    //    Creates a 4x4x4 array performs 2 transforms and then 2 inverse
    //    transforms and prints results of each stage.
    //
    //--------------------------------------------------------------------
    public static void testTransformFunctionality(){
       int dim1, dim2, dim3;

       dim1 = dim2 = dim3 = 16;

       double[][][] initArr = new double [dim1][dim2][dim3];

       double val=1.0f;
       for (int i=0; i<dim1;i++){
          for (int j=0; j<dim2;j++){
             for (int k=0; k<dim3;k++){
                initArr[i][j][k] = val++;
             }
          }
       }
       
       System.out.println(" Level = 0");
       System.out.println("==========================");

       ArrayUtil.printArrayFormatted(initArr);

       double[][][] arr2 = new double [dim1][dim2][dim3];
       double[][][] arr3 = new double [dim1][dim2][dim3];
       double[][][] arr4 = new double [dim1][dim2][dim3];
       double[][][] arr5 = new double [dim1][dim2][dim3];
       double[][][] reconArr = new double [dim1][dim2][dim3];

       System.out.println(" Level = 1");
       System.out.println("==========================");

       HaarTransform.transform(initArr, arr2, dim1, dim2, dim3 );
       ArrayUtil.printArrayFormatted(arr2);

       System.out.println(" Level = 2");
       System.out.println("==========================");

       HaarTransform.transform(arr2, arr3, dim1/2, dim2/2, dim3/2 );
       ArrayUtil.printArrayFormatted(arr3);

       System.out.println(" Level = 3");
       System.out.println("==========================");

       HaarTransform.transform(arr3, arr4, dim1/4, dim2/4, dim3/4 );
       ArrayUtil.printArrayFormatted(arr4);

       System.out.println(" Level = 4");
       System.out.println("==========================");
       HaarTransform.transform(arr4, arr5, dim1/8, dim2/8, dim3/8 );
       ArrayUtil.printArrayFormatted(arr5);


       HaarTransform.inverseTransform(arr5, arr4, dim1/8, dim2/8, dim3/8 );

       ArrayUtil.printArrayFormatted(arr4);

       HaarTransform.inverseTransform(arr4, arr3, dim1/4, dim2/4, dim3/4 );

       ArrayUtil.printArrayFormatted(arr3);

       HaarTransform.inverseTransform(arr3, arr2, dim1/2, dim2/2, dim3/2 );

       ArrayUtil.printArrayFormatted(arr2);

       HaarTransform.inverseTransform(arr2, reconArr, dim1, dim2, dim3 );

       ArrayUtil.printArrayFormatted(reconArr);

       arr5[1][0][0] = 0.0f;
       ArrayUtil.printArrayFormatted(arr5);

       HaarTransform.inverseTransform(arr5, arr4, dim1/8, dim2/8, dim3/8 );

       ArrayUtil.printArrayFormatted(arr4);

       HaarTransform.inverseTransform(arr4, arr3, dim1/4, dim2/4, dim3/4 );

       ArrayUtil.printArrayFormatted(arr3);

       HaarTransform.inverseTransform(arr3, arr2, dim1/2, dim2/2, dim3/2 );

       ArrayUtil.printArrayFormatted(arr2);

       HaarTransform.inverseTransform(arr2, reconArr, dim1, dim2, dim3 );

       ArrayUtil.printArrayFormatted(reconArr);
    }



    //--------------------------------------------------------------------
    // testForOddDimTransform
    //
    // Purpose:
    //   Ensure that no transform will be needed over an odd dimension
    //
    //  Returns:
    //        0: OK
    //       !0: BAD - need to abort
    //
    //--------------------------------------------------------------------
    public static int testForOddDimTransform(int dim1, int dim2, int dim3){

        System.out.println(" dim1 = " + dim1 + " dim2 = " + dim2
                         + " dim3 = " + dim3);

        for (int i=0;i<NUM_LVLS;i++) {
            int check1 = dim1 / ((int)(Math.pow(2,i)));
            int check2 = dim2 / ((int)(Math.pow(2,i)));
            int check3 = dim3 / ((int)(Math.pow(2,i)));

            // Make sure that each dimension is even
            if ( (check1 % 2) != 0 || (check2 % 2) != 0 || (check3 % 2) != 0) {
                return -1;   
            }
        }

        return 0;
    }

    //--------------------------------------------------------------------
    // testLinearMapper
    //
    // Purpose:
    //   Verify functionality of the linear mapper
    //
    //  Returns:
    //        NOTHING
    //
    //--------------------------------------------------------------------
    public void testLinearMapper(){
       LinearMapper lm = new LinearMapper();

       int ARR_SZ = 4;
       int NUM_BINS = 8;
       float arr[][][] = new float[ARR_SZ][ARR_SZ][ARR_SZ];

       // fill array
       float val=0.0f;
       for (int i=0;i<ARR_SZ;i++){
           for (int j=0;j<ARR_SZ;j++){
               for (int k=0;k<ARR_SZ;k++){
                  arr[i][j][k] = val;
                  val += 1.0f;
               }
           }
       }

       // map coefficients
       lm.mapCoefficients(arr, NUM_BINS);

       // lookup each coefficient
       // fill array
       val=0.0f;
       for (int i=0;i<ARR_SZ;i++){
           for (int j=0;j<ARR_SZ;j++){
               for (int k=0;k<ARR_SZ;k++){
                  val = lm.getMappedValue(arr[i][j][k]);
                  System.out.println("arr[i][j][k] = " + arr[i][j][k] + 
                                     " is mapped to <<< " + val + " >>>");
               }
           }
       }
    }


//    static void createLinearCompressionFile(String filename){
//       try {
//          FileOutputStream out = new FileOutputStream(filename);
//          DataOutputStream dout = new DataOutputStream(out);
//
//          float incr = .00390625f;
//          float val = incr;
//          for (int i=0; i<256; i++) {
//             dout.writeFloat(val);
//             val += incr;
//          }
//       }
//       catch (Exception e){
//          System.out.println("Exception when creating compression file!\n");
//          e.printStackTrace();
//          System.exit(0);
//       }
//       System.exit(0);
//    }
//
//    static void createNonlinearCompressionFile(String filename){
//
//
//                        // 20 @ 0.001
//        float arr[]={   0.001f, 0.002f, 0.003f, 0.004f, 0.005f, 0.006f, 0.007f, 0.008f, 0.009f, 0.010f,
//                        0.011f, 0.012f, 0.013f, 0.014f, 0.015f, 0.016f, 0.017f, 0.018f, 0.019f, 0.020f,
//                        // 20 @ 0.002
//                        0.022f, 0.024f, 0.026f, 0.028f, 0.030f, 0.032f, 0.034f, 0.036f, 0.038f, 0.040f,
//                        0.042f, 0.044f, 0.046f, 0.048f, 0.050f, 0.052f, 0.054f, 0.056f, 0.058f, 0.060f,
//                        // 20 @ 0.003
//                        0.063f, 0.066f, 0.069f, 0.072f, 0.075f, 0.078f, 0.081f, 0.084f, 0.087f, 0.090f,
//                        0.093f, 0.096f, 0.099f, 0.102f, 0.105f, 0.108f, 0.111f, 0.114f, 0.117f, 0.120f,
//                        // 20 @ 0.004
//                        0.124f, 0.128f, 0.132f, 0.136f, 0.140f, 0.144f, 0.148f, 0.152f, 0.156f, 0.160f,
//                        0.164f, 0.168f, 0.172f, 0.176f, 0.180f, 0.184f, 0.188f, 0.192f, 0.196f, 0.200f,
//                        // 20 @ 0.005
//                        0.205f, 0.210f, 0.215f, 0.220f, 0.225f, 0.230f, 0.235f, 0.240f, 0.245f, 0.250f,
//                        0.255f, 0.260f, 0.265f, 0.270f, 0.275f, 0.280f, 0.285f, 0.290f, 0.295f, 0.300f,
//                        // 20 @ 0.006
//                        0.306f, 0.312f, 0.318f, 0.324f, 0.330f, 0.336f, 0.342f, 0.348f, 0.354f, 0.360f,
//                        0.366f, 0.372f, 0.378f, 0.384f, 0.390f, 0.396f, 0.402f, 0.408f, 0.414f, 0.420f,
//                        // 8  @ 0.01
//                        0.430f, 0.440f, 0.450f, 0.460f, 0.470f, 0.480f, 0.490f, 0.500f,
//                        // 8  @ 0.01
//                        0.510f, 0.520f, 0.530f, 0.540f, 0.550f, 0.560f, 0.570f, 0.580f,
//                        // 20 @ 0.006
//                        0.586f, 0.592f, 0.598f, 0.604f, 0.610f, 0.616f, 0.622f, 0.628f, 0.634f, 0.640f,
//                        0.646f, 0.652f, 0.658f, 0.664f, 0.670f, 0.676f, 0.682f, 0.688f, 0.694f, 0.700f,
//                        // 20 @ 0.005
//                        0.705f, 0.710f, 0.715f, 0.720f, 0.725f, 0.730f, 0.735f, 0.740f, 0.745f, 0.750f,
//                        0.755f, 0.760f, 0.765f, 0.770f, 0.775f, 0.780f, 0.785f, 0.790f, 0.795f, 0.800f,
//                        // 20 @ 0.004
//                        0.804f, 0.808f, 0.812f, 0.816f, 0.820f, 0.824f, 0.828f, 0.832f, 0.836f, 0.840f,
//                        0.844f, 0.848f, 0.852f, 0.856f, 0.860f, 0.864f, 0.868f, 0.872f, 0.876f, 0.880f,
//                        // 20 @ 0.003
//                        0.883f, 0.886f, 0.889f, 0.892f, 0.895f, 0.898f, 0.901f, 0.904f, 0.907f, 0.910f,
//                        0.913f, 0.916f, 0.919f, 0.922f, 0.925f, 0.928f, 0.931f, 0.934f, 0.937f, 0.940f,
//                        // 20 @ 0.002
//                        0.942f, 0.944f, 0.946f, 0.948f, 0.950f, 0.952f, 0.954f, 0.956f, 0.958f, 0.960f,
//                        0.962f, 0.964f, 0.966f, 0.968f, 0.970f, 0.972f, 0.974f, 0.976f, 0.978f, 0.980f,
//                        // 20 @ 0.001
//                        0.981f, 0.982f, 0.983f, 0.984f, 0.985f, 0.986f, 0.987f, 0.988f, 0.989f, 0.990f,
//                        0.991f, 0.992f, 0.993f, 0.994f, 0.995f, 0.996f, 0.997f, 0.998f, 0.999f, 1.000f
//                  };
//
//
//       try {
//          FileOutputStream out = new FileOutputStream(filename);
//          DataOutputStream dout = new DataOutputStream(out);
//
//          for (int i=0;i<256; i++) {
//             dout.writeFloat(arr[i]);
//             System.out.println(arr[i]);
//          }
//       }
//       catch (Exception e){
//          System.out.println("Exception when creating compression file!\n");
//          e.printStackTrace();
//          System.exit(0);
//       }
//       System.exit(0);
//    }
//


    public static void print_sum(){
        int i=0;
        System.out.print("=");
        for (i=2; i<4592; i+=9) {
           System.out.print("C"+(i+9)+"+");
        }
        System.out.print("C"+(i+9));
    }


    //--------------------------------------------------------------------
    // main
    //
    //--------------------------------------------------------------------
    public static void main(String[] args) {

        TestBench2 tb = new TestBench2(3);

        // Determine endianness and get name of directory holding data
        tb.processCommandArgs(args);

        // Write to beginning of log
        String str = "Filename, description, SSE, RMSE, Normalized RMSE, "
                     + " Range, SUM_ABS_ERROR, MAX_ABS_ERROR, AVG_ABS_ERROR\n";

        try {
            System.out.println("Writing: " + str);
            log.write(str);
            log2.write(str);
        } catch (Exception e) {
            e.printStackTrace();
            System.exit(0);
        }

        // get list of all the data files in argument's directory
        ArrayList<String> testFilesList = tb.getListOfFiles();

        // test each file one by one
        tb.testFiles(testFilesList);




        // partition a file
        //Partition part = new Partition();
//        part.createPartitions("head-float.bin", 4, true);
//        part.createPartitions("jcd0004.3df.003360_384dim.rr.bin", 4, false);
        //part.createPartitions("000480-576x192x288.bin", 8, false);
        //part.createPartitions("raf512x256x256.rr.000480.bin", 32, false);

        //part.createPartitions("64partitions_0-0-0.bin", 4, false);

        //


        //FileUtil fileUtil = new FileUtil();
        //fileUtil.convert(300,200,630,288,192,576,
        //                   "thm-20070323-j13c.bin.000480.630x200x300.rr.bin",
        //                   "thm-20070323-j13c.bin.000480.576x192x288.rr.bin");

        // testing calls


//        String [] arr = {"1","2","3","4","5","6","7"};
//        CombinationGenerator.generateCombinations(arr,1,"0");

        // verification of basic code
        //tb.testTransformFunctionality();
        //tb.basicFunctionalityTest1();
        //tb.basicFunctionalityTest2(tb);
        //tb.verifyErrorReduction();

        // Run verification test on Recursive Haar Transforms
        //HaarTransform ht = new HaarTransform();
        //ht.testRecursiveWavelets();

//        // convert jimmy data dimension form 392 to 384
//        File dir = new File("../data/jimmy_timesteps_392/");
//
//        String[] children = dir.list();
//        if (children == null) {
//           System.out.println("No such directory! ");
//           System.exit(0);
//        }
//        else {
//           String filename=null;
//           for (int i=0; i<children.length; i++) {
//              // Get filename of file or directory
//              filename = children[i];
//
//              if (filename.endsWith(".bin")) {
//                 FileUtil fileUtil = new FileUtil();
//
//                 System.out.println("Copying <" +filename+ "> to "
//                                    + "384_"+filename);
//                 fileUtil.convertJimmy392to384("../data/jimmy_timesteps_392/"+filename,
//                                      "../data/jimmy_timesteps_392/"+"384_"+filename);
//              }
//           }
//       }

        

 


        // test Linear Mapper
        //tb.testLinearMapper();



    }
}



